#ifndef OMG_DDS_CORE_DETAIL_INSTANCE_HANDLE_HPP_
#define OMG_DDS_CORE_DETAIL_INSTANCE_HANDLE_HPP_

/*
 * Copyright 2010, Object Management Group, Inc.
 * Copyright 2010, PrismTech, Corp.
 * Copyright 2010, Real-Time Innovations, Inc.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include <functional>  // for std::hash

#include <dds/core/TInstanceHandle.hpp>
#include <rti/core/InstanceHandle.hpp>

namespace dds { namespace core { namespace detail {
typedef dds::core::TInstanceHandle<rti::core::InstanceHandle> InstanceHandle;
}}}  // namespace dds::core::detail

namespace std {
template <>
struct hash<dds::core::detail::InstanceHandle> {
    size_t operator()(const dds::core::detail::InstanceHandle& h) const
    {
        // elements_to_hash will be 16 / 8 = 2 for 64-bit architectures and
        // 16 / 4 = 4 for 32-bit architectures.
        constexpr size_t elements_to_hash =
                static_cast<size_t>(MIG_RTPS_KEY_HASH_MAX_LENGTH)
                / sizeof(size_t);
        const size_t *key_hash = reinterpret_cast<const size_t *>(
                &h.extensions().native().keyHash.value[0]);

        // This loop should be optimized away by most 64-bit compilers
        size_t result = 0;
        for (size_t i = 0; i < elements_to_hash; ++i) {
            result ^= key_hash[i];
        }

        return result;
    }
};

}  // namespace std

#endif /* !defined(OMG_DDS_CORE_DETAIL_INSTANCE_HANDLE_HPP_) */
