#ifndef OMG_DDS_DOMAIN_FIND_HPP_
#define OMG_DDS_DOMAIN_FIND_HPP_

/* Copyright 2010, Object Management Group, Inc.
 * Copyright 2010, PrismTech, Corp.
 * Copyright 2010, Real-Time Innovations, Inc.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include <dds/domain/DomainParticipant.hpp>

namespace dds { namespace domain {

/**
 * @relatesalso dds::domain::DomainParticipant
 *
 * @brief Locates an existing dds::domain::DomainParticipant.
 *
 * \p \#include \p <dds/domain/find.hpp>
 *
 * @namespace_note dds::domain
 *
 * If no such DomainParticipant exists, the operation will return
 * dds::core::null.
 *
 * If multiple DomainParticipants belonging to that domain id
 * exist, then the operation will return one of them. It is not
 * specified which one.
 *
 * @param domain_id The domain id of the DomainParticipant to find
 */
OMG_DDS_API
DomainParticipant find(int32_t domain_id);

} }

#endif /* OMG_DDS_DOMAIN_FIND_HPP_ */
