/* $Id$

(c) Copyright, Real-Time Innovations, 2014-2016.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.
*/

#ifndef RTI_DDS_CORE_XTYPES_DYNAMICDATAMEMBERINFO_HPP_
#define RTI_DDS_CORE_XTYPES_DYNAMICDATAMEMBERINFO_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include "dds_c/dds_c_dynamicdata.h"

#include <dds/core/types.hpp>
#include <dds/core/Value.hpp>
#include <dds/core/String.hpp>
#include <rti/core/NativeValueType.hpp>

namespace rti { namespace core { namespace xtypes {

class DynamicDataMemberInfo;
class DynamicDataMemberInfoView;

class OMG_DDS_API DynamicDataMemberInfoNativeAdapter {
public:
    typedef DDS_DynamicDataMemberInfo native_type;

    static void finalize(native_type&);
    static void copy(native_type& destination, const native_type& source);
    static void initialize(native_type& native_value);
    static bool equals(const native_type& first, const native_type& second);
};

} // namespace xtypes

typedef PODNativeAdapter<DDS_DynamicDataMemberInfo>
        DynamicDataMemberInfoViewNativeAdapter;

template <>
struct native_type_traits<xtypes::DynamicDataMemberInfoView> {
    typedef DynamicDataMemberInfoViewNativeAdapter adapter_type;
    typedef DDS_DynamicDataMemberInfo native_type;
};

template <>
struct native_type_traits<xtypes::DynamicDataMemberInfo> {
    typedef xtypes::DynamicDataMemberInfoNativeAdapter adapter_type;
    typedef DDS_DynamicDataMemberInfo native_type;
};

namespace xtypes {

/**
 * @ingroup DDSXTypesModule
 * @RTI_class_definition class DynamicDataMemberInfo
 * @brief Contains information about a DynamicData member
 *
 * @see dds::core::xtypes::DynamicData::member_info()
 */
class OMG_DDS_API DynamicDataMemberInfo :
    public NativeValueType<DynamicDataMemberInfo> {
public:
    RTI_NATIVE_VALUE_TYPE_DEFINE_DEFAULT_MOVE_OPERATIONS(DynamicDataMemberInfo)

public:
    DynamicDataMemberInfo()
    {
    }

    /**
     * @brief The member index
     *
     * @see \ref DDSDynamicDataModule_member_name_id.
     */
    uint32_t member_index() const;

    /**
     * @brief The member name
     *
     * @see \ref DDSDynamicDataModule_member_name_id.
     */
    const dds::core::string& member_name() const;

    /**
     * @brief The type kind of this member
     *
     * This a convenience function, equivalent to looking up the member's
     * DynamicType::kind().
     *
     * @see dds::core::xtypes::DynamicData::type()
     * @see dds::core::xtypes::DynamicType::kind()
     */
    dds::core::xtypes::TypeKind member_kind() const;
    uint32_t representation_count() const;

    /**
     * @brief The number of elements in this member
     *
     * Only applies to members of ArrayType and SequenceType. For any other
     * type this is always zero.
     */
    uint32_t element_count() const;

    /**
     * @brief If this member is a collection, this function returns the type kind
     * of its elements.
     *
     * This function provides the same information as
     * dds::core::xtypes::CollectionType::content_type().
     *
     * @see dds::core::xtypes::DynamicData::type()
     * @see dds::core::xtypes::CollectionType::content_type()
     */
    dds::core::xtypes::TypeKind element_kind() const;

    /**
     * @brief Indicates if this member is present in this sample
     *
     * @details \dref_details_DynamicDataMemberInfo_member_exists
     */
    bool member_exists() const;
};

// This class, unlike DynamicDataMemberInfo, doesn't make copies of the
// underlying data, so it's only valid while the DynamicData object is alive.
class OMG_DDS_API DynamicDataMemberInfoView :
    public rti::core::NativeValueType<DynamicDataMemberInfoView> {
public:
    RTI_NATIVE_VALUE_TYPE_DEFINE_DEFAULT_MOVE_OPERATIONS(DynamicDataMemberInfoView)

    uint32_t member_index() const;
    const dds::core::string& member_name() const;
    dds::core::xtypes::TypeKind member_kind() const;
    uint32_t element_count() const;
    dds::core::xtypes::TypeKind element_kind() const;
    bool member_exists() const;

    DynamicDataMemberInfo clone() const;
};

} } } // namespace rti::core::xtypes

#endif // RTI_DDS_CORE_XTYPES_DYNAMICDATAMEMBERINFO_HPP_
