/* $Id$

(c) Copyright, Real-Time Innovations, 2018.
All rights reserved.

No duplications, whole or partial, manual or electronic, may be made
without express written permission.  Any such copies, or
revisions thereof, must display this notice unaltered.
This code contains trade secrets of Real-Time Innovations, Inc.

============================================================================= */

// This file contains helpers for the interpreted type plugin
// generated by rtiddsgen

#ifndef RTI_DDS_TOPIC_CDR_INTERPRETER_HELPERS_HPP_
#define RTI_DDS_TOPIC_CDR_INTERPRETER_HELPERS_HPP_

// IMPORTANT: macros.hpp must be the first RTI header included in every header
// file so that symbols are exported correctly on Windows
#include <dds/core/macros.hpp>

#include <string>
#include <wchar.h>
#include "xcdr/xcdr_typeCode.h"
#include "dds_c/dds_c_typecode.h"
#include "osapi/osapi_utility.h"

#include "rti/xcdr/Interpreter.hpp"

namespace rti { namespace topic { namespace interpreter {

// This is a macro and not a template function because the function may not be
// inlined by some compilers
#define RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER( \
    T, \
    sample, \
    member_ptr, \
    binding_member_offset, \
    member_tc, \
    member_info, \
    allocate_member_if_null) \
{ \
    if (member_info != NULL && \
            RTIXCdrTypeCodeMember_isOptional(member_info)) \
    { \
        T **memberPtrPtr = NULL; \
        memberPtrPtr = reinterpret_cast<T **>( \
                ((char *)sample) + binding_member_offset); \
        member_ptr = *memberPtrPtr; \
 \
        if (member_ptr == NULL) { \
            if (allocate_member_if_null) { \
                const RTIXCdrTypeCode *tc = member_tc; \
                if (tc != NULL && RTIXCdrTypeCode_getKind(tc) == RTI_XCDR_TK_ARRAY) { \
                    size_t count = RTIXCdrTypeCode_getArrayElementCount(tc); \
                    *memberPtrPtr = \
                            rti::core::memory::ObjectAllocator<T>::create_array(count); \
                } else { \
                    *memberPtrPtr = \
                            rti::core::memory::ObjectAllocator<T>::create(); \
                } \
                member_ptr = *memberPtrPtr; \
            } \
        } \
    } else { \
        member_ptr = reinterpret_cast<T *>( \
                (((char *)sample) + binding_member_offset)); \
    } \
}

// --- Interpreter callbacks: -------------------------------------------------

namespace detail {

template <typename SequenceType>
static size_t sequence_length(const SequenceType &sequence) {
    return sequence.size();
}

inline
size_t sequence_length(const std::string &sequence) {
    return sequence.empty() ? 1 : strlen(sequence.c_str()) + 1;
}

inline
size_t sequence_length(const std::wstring &sequence) {
    return sequence.empty() ? 1 : wcslen(sequence.c_str()) + 1;
}

template <typename SequenceType, typename Enable = void>
struct sequence_resize {
    static void resize(SequenceType& sequence, size_t size)
    {
        sequence.resize(size);
    }
};

// *QNX7.0.0qcc_cxx5.4.0 doesn't have a functioning std::allocator_traits
#if !(defined(RTI_QNX7) && defined(_LIBCPP_VERSION) && _LIBCPP_VERSION == 3700)
template<typename T>
struct sequence_resize<
        std::vector<T>,
        typename std::enable_if<
                std::is_trivial<T>::value
                && std::is_standard_layout<T>::value
                && !std::is_same<T, bool>::value>::type> {
    static void resize(std::vector<T> &sequence, size_t size)
    {
        // For std::vector<T> such that T is a POD (a primitive type in our
        // case), avoid initializing the elements on resize(). Note that bool
        // is explicitly excluded from this optimization because
        // std::vector<bool> has a specialized implementation.

        // Declare a struct with the same size as T but a no-op default
        // constructor
        struct no_init {
            T v;
            no_init()
            {
                // empty
                // ROBUSTNESS-253
                /* coverity[uninit_member] */
            }
        };

        static_assert(
                sizeof(no_init[4]) == sizeof(T[4]),
                "T and no_init do not have the same alignment");

        // Recast std::vector<T> as an equivalent vector whose alocator doesn't
        // initialize its members.
        using allocator = typename std::vector<T>::allocator_type;
        using no_init_allocator = typename std::allocator_traits<
                allocator>::template rebind_alloc<no_init>;
        using no_init_vector = std::vector<no_init, no_init_allocator>;

        reinterpret_cast<no_init_vector &>(sequence).resize(size);
    }
};
#endif

template <typename T, size_t N>
struct sequence_resize<rti::core::bounded_sequence<T, N> > {
    static void resize(rti::core::bounded_sequence<T, N>& seq, size_t size)
    {
        seq.resize_uninitialized(size);
    }
};

template <typename CharT>
struct sequence_resize<std::basic_string<CharT> > {
    static void resize(std::basic_string<CharT>& seq, size_t size)
    {
        seq.resize(size - 1);
    }
};

template <
    typename SequenceType,
    typename ElementType = typename SequenceType::value_type>
struct sequence_helper {
    // The interpreter calls this function to obtain the pointer to the
    // sequence data
    static RTIXCdrMemberValue get_value_pointer(
        void *sample,
        RTIXCdrUnsignedLong *element_count,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean,
        void *)
    {
        // Important: this pointer must be const (CORE-11649). The interpreter
        // is guaranteed to call this function for read-only access.
        const SequenceType *sequence;
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            SequenceType,
            sample,
            sequence,
            binding_member_offset,
            NULL,
            member_info,
            // With optional sequences there is no memory allocation
            // in this codepath
            RTI_XCDR_FALSE);

        if (sequence == NULL) {
            return_value.isNull = RTI_XCDR_TRUE;
            return return_value;
        }

        if (element_count != NULL) {
            *element_count = static_cast<RTIXCdrUnsignedLong>(
                    sequence_length(*sequence));
        }

        return_value.value.ptr = sequence->empty() ? 
                NULL : 
                (char *) &(*sequence)[0];
        return return_value;
    }

    // The interpreter calls this function to resize a sequence
    static RTIXCdrMemberValue set_element_count(
            RTIXCdrBoolean *error,
            void *sample,
            RTIXCdrUnsignedLong element_count,
            RTIXCdrUnsignedLongLong binding_member_offset,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            RTIXCdrBoolean,
            RTIXCdrBoolean,
            void *)
    {
        *error = 0;
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

        try {
            SequenceType *sequence;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                SequenceType,
                sample,
                sequence,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (sequence == NULL) {
                return_value.isNull = RTI_XCDR_TRUE;
                return_value.value.ptr = NULL;
                return return_value;
            }

            sequence_resize<SequenceType>::resize(*sequence, element_count);

            return_value.value.ptr = sequence->empty() ? 
                    NULL :
                    (char *) &(*sequence)[0];
            return return_value;
        } catch (...) {
            *error = 1;
            return return_value;
        }
    }

    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        static RTIXCdrSampleAccessInfo the_sample_access_info = {
                RTI_XCDR_TYPE_BINDING_CPP_03_STL,
                {sizeof(SequenceType),0,0,0},
                RTI_XCDR_FALSE,
                get_value_pointer,
                set_element_count,
                NULL,
                NULL,
                NULL
        };

        return &the_sample_access_info;
    }

};

// sequences of booleans require a special treatment because std::vector<bool>'s
// specialized implementation doesn't contain a bool*.
template <typename SequenceType>
struct sequence_helper<SequenceType, bool> {

    // The interpreter calls this function to obtain the pointer to the
    // sequence data
    static RTIXCdrMemberValue get_value_pointer(
        void *sample,
        RTIXCdrUnsignedLong *element_count,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong index,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean,
        void *)
    {
        // Important: this pointer must be const (CORE-11649). The interpreter
        // is guaranteed to call this function for read-only access.
        const SequenceType *sequence;
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            SequenceType,
            sample,
            sequence,
            binding_member_offset,
            NULL,
            member_info,
            // With optional sequences there is no memory allocation
            // in this codepath
            RTI_XCDR_FALSE);

        if (sequence == NULL) {
            return_value.isNull = RTI_XCDR_TRUE;
            return return_value;
        }

        if (element_count != NULL) {
            *element_count = static_cast<RTIXCdrUnsignedLong>(
                    sequence_length(*sequence));
        }

        return_value.value.bVal = sequence->empty() ? 0 : (*sequence)[index];
        return return_value;
    }

    // The interpreter calls this function to resize a sequence
    static RTIXCdrMemberValue set_element_count(
            RTIXCdrBoolean *error,
            void *sample,
            RTIXCdrUnsignedLong element_count,
            RTIXCdrUnsignedLongLong binding_member_offset,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            RTIXCdrBoolean,
            RTIXCdrBoolean,
            void *)
    {
        *error = 0;
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;
        try {
            SequenceType *sequence;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                SequenceType,
                sample,
                sequence,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (sequence == NULL) {
                return_value.isNull = RTI_XCDR_TRUE;
                return_value.value.ptr = NULL;
                return return_value;
            }

            sequence_resize<SequenceType>::resize(*sequence, element_count);

            return_value.value.bVal = sequence->empty() ? 0 : (*sequence)[0];
            return return_value;
        } catch (...) {
            *error = 1;
            return return_value;
        }
    }

    static void set_element_value(
        void *sample,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong index,
        RTIXCdrMemberValue value,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean,
        void *)
    {
        SequenceType *sequence;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            SequenceType,
            sample,
            sequence,
            binding_member_offset,
            NULL,
            member_info,
            // With optional sequences there is no memory allocation
            // in this codepath
            RTI_XCDR_FALSE);

        if (sequence == NULL) {
            return;
        }

        (*sequence)[index] = value.value.bVal ? true : false;
    }

    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        static RTIXCdrSampleAccessInfo the_sample_access_info = {
                RTI_XCDR_TYPE_BINDING_CPP_03_STL,
                {sizeof(SequenceType),0,0,0},
                RTI_XCDR_FALSE,
                get_value_pointer,
                set_element_count,
                set_element_value,
                NULL,
                NULL
        };

        return &the_sample_access_info;
    }
};

// Use this specialization for sequences of wchar if sizeof(wchar) != 2
// If sizeof(wchar) is 2, this can be treated as any other sequence of
// primitives, and the general template works.
template <typename SequenceType>
struct sequence_helper<
    SequenceType,
    typename std::enable_if<
        std::is_same<typename SequenceType::value_type, wchar_t>::value
            && sizeof(typename SequenceType::value_type) != 2,
        typename SequenceType::value_type>::type
    > {
    // The interpreter calls this function to obtain the pointer to the
    // sequence data
    static RTIXCdrMemberValue get_value_pointer(
        void *sample,
        RTIXCdrUnsignedLong *element_count,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong index,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean,
        void *)
    {
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;
        const SequenceType *sequence;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            SequenceType,
            sample,
            sequence,
            binding_member_offset,
            NULL,
            member_info,
            // With optional sequences there is no memory allocation
            // in this codepath
            RTI_XCDR_FALSE);

        if (sequence == NULL) {
            return_value.isNull = RTI_XCDR_TRUE;
            return return_value;
        }

        if (element_count != NULL) {
            *element_count = static_cast<RTIXCdrUnsignedLong>(
                    sequence_length(*sequence));
        }

        return_value.value.wVal = static_cast<RTIXCdrWchar>(
                sequence->empty() ? 0 : (*sequence)[index]);
        return return_value;
    }

    // The interpreter calls this function to resize a sequence
    static RTIXCdrMemberValue set_element_count(
            RTIXCdrBoolean *error,
            void *sample,
            RTIXCdrUnsignedLong element_count,
            RTIXCdrUnsignedLongLong binding_member_offset,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            RTIXCdrBoolean,
            RTIXCdrBoolean,
            void *)
    {
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;
        *error = 0;

        try {
            SequenceType *sequence;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                SequenceType,
                sample,
                sequence,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (sequence == NULL) {
                return_value.isNull = RTI_XCDR_TRUE;
                return return_value;
            }

            sequence_resize<SequenceType>::resize(*sequence, element_count);

            return_value.value.wVal = static_cast<RTIXCdrWchar>(
                    sequence->empty() ? 0 : (*sequence)[0]);
            return return_value;
        } catch (...) {
            *error = 1;
            return return_value;
        }
    }

    static void set_element_value(
        void *sample,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong index,
        RTIXCdrMemberValue value,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean,
        void *)
    {
        SequenceType *sequence;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            SequenceType,
            sample,
            sequence,
            binding_member_offset,
            NULL,
            member_info,
            // With optional sequences there is no memory allocation
            // in this codepath
            RTI_XCDR_FALSE);

        if (sequence == NULL) {
            return;
        }

        if (index >= sequence->size()) { // ignore null terminator
            return;
        }

        (*sequence)[index] = value.value.wVal;
    }

    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        static RTIXCdrSampleAccessInfo the_sample_access_info = {
                RTI_XCDR_TYPE_BINDING_CPP_03_STL,
                {sizeof(SequenceType),0,0,0},
                RTI_XCDR_FALSE,
                get_value_pointer,
                set_element_count,
                set_element_value,
                NULL,
                NULL
        };

        return &the_sample_access_info;
    }
};

template <size_t SizeOfWchar>
struct wchar_helper {

    static RTIXCdrMemberValue get_value_pointer(
            void *sample,
            RTIXCdrUnsignedLong *,
            RTIXCdrUnsignedLongLong binding_member_offset,
            RTIXCdrUnsignedLong index,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            void *)
    {
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

        try {
            wchar_t* field;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                wchar_t,
                sample,
                field,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (field == NULL) {
                return_value.isNull = RTI_XCDR_TRUE;
                return return_value;
            }

            return_value.value.wVal = (RTIXCdrWchar) field[index];
            return return_value;
        } catch (...) {
            return_value.value.wVal = 0;
            return return_value;
        }
    }

    static void set_element_value(
            void *sample,
            RTIXCdrUnsignedLongLong binding_member_offset,
            RTIXCdrUnsignedLong index,
            RTIXCdrMemberValue value,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            void *)
    {
        try {
            wchar_t* field;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                wchar_t,
                sample,
                field,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (field == NULL) {
                return;
            }

            field[index] = value.value.wVal;
            return;
        } catch (...) {
            return;
        }
    }

    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        static RTIXCdrSampleAccessInfo the_sample_access_info = {
                RTI_XCDR_TYPE_BINDING_CPP_03_STL,
                {sizeof(wchar_t),0,0,0},
                RTI_XCDR_FALSE,
                get_value_pointer,
                NULL,
                set_element_value,
                NULL,
                NULL
        };

        return &the_sample_access_info;
    }
};

template <>
struct wchar_helper<2> {
    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        return NULL;
    }
};

template <size_t SizeOfBool>
struct bool_helper {

    static RTIXCdrMemberValue get_value_pointer(
            void *sample,
            RTIXCdrUnsignedLong *,
            RTIXCdrUnsignedLongLong binding_member_offset,
            RTIXCdrUnsignedLong index,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            void *)
    {
        RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

        try {
            const bool *field;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                bool,
                sample,
                field,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (field == NULL) {
                return_value.isNull = RTI_XCDR_TRUE;
                return return_value;
            }

            return_value.value.bVal = field[index];
            return return_value;
        } catch (...) {
            return_value.value.bVal = RTI_XCDR_FALSE;
            return return_value;
        }
    }

    static void set_element_value(
            void *sample,
            RTIXCdrUnsignedLongLong binding_member_offset,
            RTIXCdrUnsignedLong index,
            RTIXCdrMemberValue value,
            const struct RTIXCdrTypeCode *,
            const struct RTIXCdrTypeCodeMember *member_info,
            RTIXCdrBoolean allocate_member_if_null,
            void *)
    {
        try {
            bool* field;

            RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
                bool,
                sample,
                field,
                binding_member_offset,
                NULL,
                member_info,
                allocate_member_if_null);

            if (field == NULL) {
                return;
            }

            field[index] = value.value.bVal;
            return;
        } catch (...) {
            return;
        }
    }

    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        static RTIXCdrSampleAccessInfo the_sample_access_info = {
                RTI_XCDR_TYPE_BINDING_CPP_03_STL,
                {sizeof(bool),0,0,0},
                RTI_XCDR_FALSE,
                get_value_pointer,
                NULL,
                set_element_value,
                NULL,
                NULL
        };

        return &the_sample_access_info;
    }
};

template <>
struct bool_helper<1> {
    static RTIXCdrSampleAccessInfo * sample_access_info()
    {
        return NULL;
    }
};

} // namespace detail

template <typename AggregationType>
static RTIXCdrMemberValue get_aggregation_value_pointer(
        void *sample,
        RTIXCdrUnsignedLong *,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong,
        const struct RTIXCdrTypeCode *,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean allocate_member_if_null,
        void *)
{
    RTIXCdrMemberValue return_value = RTIXCdrMemberValue_INITIALIZER;

    try {
        AggregationType *valuePtr;

        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            AggregationType,
            sample,
            valuePtr,
            binding_member_offset,
            NULL,
            member_info,
            allocate_member_if_null);

        if (valuePtr == NULL) {
            return_value.isNull = RTI_XCDR_TRUE;
            return return_value;
        }

        return_value.value.ptr = reinterpret_cast<char *>(valuePtr);
        return return_value;
    } catch(...) {
        return_value.value.ptr = NULL;
        return return_value;
    }
}

template <typename ArrayType>
static RTIXCdrMemberValue get_opt_array_value_pointer(
        void *sample,
        RTIXCdrUnsignedLong *,
        RTIXCdrUnsignedLongLong binding_member_offset,
        RTIXCdrUnsignedLong,
        const struct RTIXCdrTypeCode *member_tc,
        const struct RTIXCdrTypeCodeMember *member_info,
        RTIXCdrBoolean allocate_member_if_null,
        void *)
{
    RTIXCdrMemberValue return_value = RTI_XCDR_MEMBER_VALUE_NIL;

    try {
        ArrayType *valuePtr;

        /**
         * Coverity is complaining because we are casting a wide character
         * operand valuePtr into narrow character. It is true when ArrayType
         * is a wchar_t, this being said, the code is correct because what
         * the interpreter is expecting is just a pointer, the content of
         * the array will be interpreted later, as a wide character, if that's
         * the case.
         */
        /* coverity[cert_str38_c_violation : FALSE] */
        RTI_TOPIC_INTERPRETER_GET_MEMBER_VALUE_POINTER(
            ArrayType,
            sample,
            valuePtr,
            binding_member_offset,
            member_tc,
            member_info,
            allocate_member_if_null);

        if (valuePtr == NULL) {
            return return_value;
        }

        /* coverity[cert_str38_c_violation : FALSE] */
        return_value.value.ptr = reinterpret_cast<char *>(valuePtr);
        return_value.isNull = RTI_XCDR_FALSE;
        return return_value;
    } catch(...) {
        return return_value;
    }
}


template <typename T>
static RTIXCdrSampleAccessInfo * get_opt_array_sample_access_info()
{
    static RTIXCdrSampleAccessInfo the_sample_access_info = {
            RTI_XCDR_TYPE_BINDING_CPP_03_STL,
            {sizeof(T),0,0,0},
            RTI_XCDR_FALSE,
            get_opt_array_value_pointer<T>,
            NULL,
            NULL,
            NULL,
            NULL
    };

    return &the_sample_access_info;
}


inline char * get_external_value_pointer(void *ref)
{
    dds::core::external<char> *ref_as_external = 
            reinterpret_cast<dds::core::external<char> *>(ref);
    return ref_as_external->get();
}

template <typename SequenceType>
DDS_TypeCode initialize_sequence_typecode(RTICdrUnsignedLong bound)
{
    static const DDS_TypeCode INITIALIZER = DDS_INITIALIZE_SEQUENCE_TYPECODE(0, NULL);

    DDS_TypeCode type_code = INITIALIZER;

    type_code._data._maximumLength = bound;
    type_code._data._sampleAccessInfo =
            detail::sequence_helper<SequenceType>::sample_access_info();
    return type_code;
}

inline DDS_TypeCode initialize_string_typecode(RTICdrUnsignedLong bound)
{
    static const DDS_TypeCode INITIALIZER = DDS_INITIALIZE_STRING_TYPECODE(0);

    DDS_TypeCode type_code = INITIALIZER;

    type_code._data._maximumLength = bound;
    type_code._data._sampleAccessInfo =
            detail::sequence_helper<std::string>::sample_access_info();
    return type_code;
}

inline DDS_TypeCode initialize_wstring_typecode(RTICdrUnsignedLong bound)
{
    static const DDS_TypeCode INITIALIZER = DDS_INITIALIZE_WSTRING_TYPECODE(0);

    DDS_TypeCode type_code = INITIALIZER;

    type_code._data._maximumLength = bound;
    type_code._data._sampleAccessInfo =
            detail::sequence_helper<std::wstring>::sample_access_info();
    return type_code;
}

inline const DDS_TypeCode& initialize_wchar_typecode()
{
    static DDS_TypeCode type_code = DDS_g_tc_wchar;
    type_code._data._sampleAccessInfo =
            detail::wchar_helper<sizeof(wchar_t)>::sample_access_info();
    return type_code;
}

inline const DDS_TypeCode& initialize_bool_typecode()
{
    static DDS_TypeCode type_code = DDS_g_tc_boolean;
    type_code._data._sampleAccessInfo =
            detail::bool_helper<sizeof(bool)>::sample_access_info();
    return type_code;
}

// Interpreter Programs singleton (for to/from_cdr functions)

namespace detail {

struct PropertyConfigurator {
    static void configure(RTIXCdrInterpreterProgramsGenProperty& property)
    {
        property.externalReferenceSize = 
                (RTIXCdrUnsignedShort) sizeof(dds::core::external<char>);
        property.getExternalRefPointerFcn = 
                rti::topic::interpreter::get_external_value_pointer;
    }
};

}

// Runs the general rti::xcdr::get_cdr_serialization_programs but customizes
// the property via PropertyConfigurator.
template <
    typename TopicType, 
    bool ResolveAlias, 
    bool InlineStruct, 
    bool OptimizeEnum>
RTIXCdrInterpreterPrograms * get_cdr_serialization_programs()
{
    return rti::xcdr::get_cdr_serialization_programs_w_property_configurator<
            TopicType,
            ResolveAlias,
            InlineStruct,
            OptimizeEnum,
            detail::PropertyConfigurator>();
}

template <
    typename TopicType,
    bool ResolveAlias = true,
    bool InlineStruct = true,
    bool OptimizeEnum = true>
void regenerate_cdr_serialization_programs()
{
    rti::core::check_create_entity(
            rti::xcdr::ProgramsSingleton<
                    TopicType,
                    RTI_XCDR_SER_PROGRAM | RTI_XCDR_DESER_PROGRAM
                            | RTI_XCDR_GET_SER_SIZE_PROGRAM
                            | RTI_XCDR_GET_MAX_SER_SIZE_PROGRAM,
                    ResolveAlias,
                    InlineStruct,
                    OptimizeEnum,
                    false,
                    detail::PropertyConfigurator>::regenerate_programs(),
            "regenerated programs");
}

} } }

namespace rti { namespace xcdr {

// The xcdr flat data implementation needs type_code to be defined.
// Here, in the modern C++ API we can provide a general implementation.
// In the traditional C++ API type_code is defined for each type in generated
// code.
template <typename TopicType>
struct type_code {
    static const RTIXCdrTypeCode * get()
    {
        return reinterpret_cast<const RTIXCdrTypeCode *>(
                &rti::topic::dynamic_type<TopicType>::get().native());
    }
};

} }

#endif // RTI_DDS_TOPIC_CDR_INTERPRETER_HELPERS_HPP_
