/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)netio_common.h    generated by: makeheader    Sat Mar  9 09:38:15 2024
 *
 *		built from:	common.ifc
 */

#ifndef netio_common_h
#define netio_common_h


  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif
  #ifndef osapi_ntptime_h
    #include "osapi/osapi_ntptime.h"
  #endif
  #ifndef reda_buffer_h
    #include "reda/reda_buffer.h"
  #endif
  #ifndef reda_inlineList_h
    #include "reda/reda_inlineList.h"
  #endif
#ifndef reda_skiplist_h
  #include "reda/reda_skiplist.h"
#endif
#ifndef netio_cap_include_manager_h
  #include "netio_cap/netio_cap_manager.h"
#endif
#ifndef netio_dll_h
  #include "netio/netio_dll.h"
#endif
#ifndef transport_interface_h
  #include "transport/transport_interface.h"
#endif
#ifndef cdr_encapsulation_h
  #include "cdr/cdr_encapsulation.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif

struct REDAWorker; /* forward declare to appease compiler */

struct RTINetioConfigurator; /* forward declare to appease compiler */

struct RTINetioAddressFilter {
    /*e
     * Address portion of the filter.  Only the bits that correspond to
     * the "1" of the mask bit matter.
    */
    NDDS_Transport_Address_t address;
    /*e mask for the address */
    NDDS_Transport_Address_t mask;
};

extern RTINetioDllExport void
RTINetioAddressFilter_print(const struct RTINetioAddressFilter *address,
			    const char *desc, 
			    int indent);

extern RTINetioDllExport
int RTINetioAddressFilter_compare(const struct RTINetioAddressFilter *l, 
				  const struct RTINetioAddressFilter *r);

extern RTINetioDllExport
RTIBool RTINetioAddressFilter_filter(const struct RTINetioAddressFilter *me,
				     const NDDS_Transport_Address_t *address);


#define RTI_NETIO_ADDRESS_INVALID { \
      { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0} }


#define RTI_NETIO_ADDRESS_FILTER_ANY {\
      RTI_NETIO_ADDRESS_INVALID, \
      RTI_NETIO_ADDRESS_INVALID}


#define RTI_NETIO_ADDRESS_FILTER_UNSPECIFIED { \
      RTI_NETIO_ADDRESS_INVALID, \
      {{0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF, \
                0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF}} }


#define RTI_NETIO_ADDRESS_FILTER_LOOPBACK { \
      {{0, 0, 0, 1}}, \
      {{0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF, \
                0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF}} }                                                           


#define RTI_NETIO_ADDRESS_FILTER_MULTICAST { \
      {{0xFF,0x00,0x00,0x00, 0x00,0x00,0x00,0x00, \
                0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00}}, \
      {{0xFF,0x00,0x00,0x00, 0x00,0x00,0x00,0x00, \
                0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00}} }                                                                   


#define RTI_NETIO_ADDRESS_FILTER_SITE_LOCAL_UNICAST { \
      {{0xFE,0xC0,0x00,0x00, 0x00,0x00,0x00,0x00, \
                0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00}}, \
      {{0xFF,0xC0,0x00,0x00, 0x00,0x00,0x00,0x00, \
                0x00,0x00,0x00,0x00, 0x00,0x00,0x00,0x00}} }                                                                     

extern RTINetioDllExport RTIBool
RTINetioAddressFilter_from(struct RTINetioAddressFilter   *me, 
			   const NDDS_Transport_Address_t *address_range_in,
			   int transport_address_bit_count_in);


#define RTI_NETIO_ALIAS_LIST_LENGTH_MAX (128)

struct RTINetioAliasList {
    /*e @brief An NULL terminated string of characters. 
      
        One extra element, to guarantee that it is always 
        terminated by a '\0' character. Thus strncpy()
        can be safely used:
        
        \code
        RTIAliasList aliasList = RTI_NETIO_ALIAS_LIST_DEFAULT;
        strncpy(aliasList.element, sourceAliasString, 
                RTI_NETIO_ALIAS_LIST_LENGTH_MAX);
       \endcode
     */
    char element[RTI_NETIO_ALIAS_LIST_LENGTH_MAX+1]; 
};

extern RTINetioDllExport void 
RTINetioAliasList_init(struct RTINetioAliasList* me);

extern RTINetioDllExport void 
RTINetioAliasList_fromString(struct RTINetioAliasList* me, 
                             const char* stringValueIn);

extern RTINetioDllExport 
void RTINetioAliasList_fromLocatorString(
        struct RTINetioAliasList *me,
        const char *locatorString);

extern RTINetioDllExport RTIBool 
RTINetioAliasList_fromStringArray(
            struct RTINetioAliasList *me, 
            const char **stringArray,
            int stringArrayLength);

extern RTINetioDllExport const char* 
RTINetioAliasList_toString(struct RTINetioAliasList* me);

extern RTINetioDllExport RTIBool 
RTINetioAliasList_append(struct RTINetioAliasList* me, 
                         const struct RTINetioAliasList* listIn);

extern RTINetioDllExport RTIBool 
RTINetioAliasList_appendFromString(struct RTINetioAliasList* me, 
                         const char *stringIn);

extern RTINetioDllExport RTIBool
RTINetioAliasList_split(const struct RTINetioAliasList* me, 
                        struct RTINetioAliasList* headOut, 
                        struct RTINetioAliasList* tailOut);

extern RTINetioDllExport RTI_INT32 
RTINetioAliasList_count(const struct RTINetioAliasList* me);

extern RTINetioDllExport RTIBool 
RTINetioAliasList_match(struct RTINetioAliasList* candidateListIn, 
                        struct RTINetioAliasList* targetListIn);

extern RTINetioDllExport int 
RTINetioAliasList_compare(const struct RTINetioAliasList *l,
			  const struct RTINetioAliasList *r);

extern RTINetioDllExport void 
RTINetioAliasList_print(const struct RTINetioAliasList *data, 
			const char *desc, int indent);


#define RTI_NETIO_ALIAS_LIST_SEPARATOR   ','


#define RTI_NETIO_ALIAS_LIST_DEFAULT { {\
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_UDPv4 { {\
    'u', 'd', 'p', 'v', '4', 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}

#define RTI_NETIO_ALIAS_LIST_UDPv4_WAN { { \
    'u', 'd', 'p', 'v', '4', '_', 'w', 'a', 'n', 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_UDPv6 { {\
    'u', 'd', 'p', 'v', '6', 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_INTRA { {\
    'i', 'n', 't', 'r', 'a', 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_SHMEM { {\
    's', 'h', 'm', 'e', 'm', 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_STARFABRIC { {\
    's', 't', 'a', 'r', 'f', 'a', 'b', 'r', 'i', 'c', \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


#define RTI_NETIO_ALIAS_LIST_INTRA_UDPv4 { {\
    'i', 'n', 't', 'r', 'a', RTI_NETIO_ALIAS_LIST_SEPARATOR, 'u', 'd', 'p', 'v', \
    '4', 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0 /* Guarantee NULL terminated */ \
}}


/* This maximum is obtained from the RTPS spec:
 * 9.4.2.10 LocatorList: The PSM mapping for the LocatorList SubmessageElement
 * defined in Section 8.3.5.11 is given by the following IDL definition:
 *    typedef sequence<Locator_t, 8> LocatorList;
 */
#define RTPS_LOCATOR_LIST_MAX_SIZE   (8)

/* This is the current maximum number of locators RTI Connext DDS supports.
 * This is a hard limit used for preallocating multiple structures across the
 * whole tree. Also, it is used as a hard limit for multiple checks we perform
 * in higher modules. Note that while this defines a hard limit, a should limit
 * can be imposed by the middleware configuration.
 *
 * Right now, we have decided to set this to two times the RTPS limit. This
 * should provide plenty locators for supporting three interfaces while enabling
 * three ip transports plus shared memory.
 */
#define RTI_NETIO_RTPS_LOCATOR_LIST_MAX_SIZE   (2*RTPS_LOCATOR_LIST_MAX_SIZE)

/*
 * Maximum used for lists that don't need more that a couple of locators (e.g.,
 * multicast locators). Used to reduce memory usage.
 */
#define RTI_NETIO_RTPS_LOCATOR_REDUCED_LIST_MAX_SIZE   (RTPS_LOCATOR_LIST_MAX_SIZE/2)


#define RTI_NETIO_RTPS_LOCATOR_LIST_DEFAULT { \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID  \
}

#define RTI_NETIO_RTPS_LOCATOR_REDUCED_LIST_DEFAULT { \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID, \
         RTI_NETIO_LOCATOR_INVALID  \
}



#define NETIO_LOCATOR_ENCAPSULATION_COUNT_MAX (4)
#define NETIO_LOCATOR_ENCAPSULATION_ARRAY_INVALID {0, 0, 0, 0}

struct RTINetioLocator {
    /*e
      @brief A transport class.
     */
    NDDS_Transport_ClassId_t transport;

    /*e
      @brief An address. Must be unique for the transport class
      in the domain.
     */
    NDDS_Transport_Address_t address;

    /*e
     @brief Address hostname
     */
    char *hostname;

    /*e
      @brief A port number.
     */
    NDDS_Transport_Port_t port;

    /*e
      @brief Number of encapsulations
    */
    RTI_UINT32 encapsulationCount;

    /*e
      @brief encapsulation IDs
     */
    RTIEncapsulationId encapsulations[NETIO_LOCATOR_ENCAPSULATION_COUNT_MAX];

    /*e
      @brief loss probability
     */
    RTI_FLOAT32 lossProbability;

    /*e
     * @brief Flag that indicates whether a locator is disabled.
     */
    RTIBool disabled;
};


#define NDDS_TRANSPORT_MAX_HOSTNAME_LENGTH (256)

extern RTINetioDllExport
void RTINetioLocator_from(struct RTINetioLocator *me, 
		     NDDS_Transport_ClassId_t             transportIn,
		     const NDDS_Transport_Address_t      *addressIn,
		     NDDS_Transport_Port_t                portIn);

extern RTINetioDllExport 
void RTINetioLocator_print(const struct RTINetioLocator *me, 
			   const char *desc,
			   int indent);

extern RTINetioDllExport 
void RTINetioLocator_printODT(void *me, 
			   const char *desc,
			   int indent);


#define RTI_NETIO_LOCATOR_STRING_BUFFER_SIZE \
    RTI_NETIO_ALIAS_LIST_LENGTH_MAX + 3 /*strlen("://")*/ + \
    NDDS_TRANSPORT_ADDRESS_STRING_BUFFER_SIZE + 6/*strlen(":99999")*/ + 1

extern RTINetioDllExport 
RTIBool RTINetioLocator_toString(
    const struct RTINetioLocator *me, 
    const struct RTINetioAliasList * aliasList,
    char * buffer,
    RTI_UINT32 bufferSize);

extern RTINetioDllExport
int RTINetioLocator_compare(const struct RTINetioLocator *l, 
			    const struct RTINetioLocator *r);

extern RTINetioDllExport
int RTINetioLocator_compareWithEncapsulationODT(void *l, void *r);

extern RTINetioDllExport
int RTINetioLocator_compareWithEncapsulation(const struct RTINetioLocator *l, 
                                             const struct RTINetioLocator *r);

struct RTINetioLocatorCompareOptions {
    RTIBool compareEncapsulations;
    RTIBool compareHostname;
    RTIBool compareAddress;
    RTIBool comparePort;
};


#define RTI_NETIO_LOCATOR_COMPARE_OPTIONS_DEFAULT \
  { \
    RTI_FALSE, /* compareEncapsulations */ \
    RTI_FALSE, /* compareHostname */ \
    RTI_FALSE, /* compareAddress*/ \
    RTI_FALSE  /* comparePort */ \
  }

extern RTINetioDllExport
int RTINetioLocator_compareWithCompareOptions(
        const struct RTINetioLocator *l,
        const struct RTINetioLocator *r,
        const struct RTINetioLocatorCompareOptions *compareOptions);

extern RTINetioDllExport
void RTINetioLocator_addEncapsulations(
                    struct RTINetioLocator *me, 
                    int encapsulationCount,
                    RTIEncapsulationId *encapsulations,
                    RTIBool *foundMoreThanProvidedFor);

extern RTINetioDllExport
void RTINetioLocator_initialize(struct RTINetioLocator *me);

extern RTINetioDllExport
void RTINetioLocator_finalize(
        struct RTINetioLocator *me);

extern RTINetioDllExport
RTIBool RTINetioLocator_copy(
        struct RTINetioLocator *me,
        const struct RTINetioLocator *base);


#define RTI_NETIO_LOCATOR_INVALID \
  { \
    NDDS_TRANSPORT_CLASSID_INVALID,            /* transportClass */ \
    RTI_NETIO_ADDRESS_INVALID,                 /* address */ \
    NULL,                                      /* hostname */ \
    0,                                         /* port */ \
    0,                                         /* encapsulationCount */ \
    NETIO_LOCATOR_ENCAPSULATION_ARRAY_INVALID, /* encapsulationIDs */ \
    0.0,                                       /* lossProbability */ \
    RTI_FALSE                                  /* disabled */ \
  }

struct RTINetioLocatorInlineNode {
    struct REDAInlineListNode parent;
    struct RTINetioLocator self;
};


#define RTINetioLocatorInlineNode_INITIALIZER \
  { \
    REDAInlineListNode_INITIALIZER, /* parent */ \
    RTI_NETIO_LOCATOR_INVALID       /* self */ \
  }

struct REDAFastBufferPool;

extern RTINetioDllExport struct RTINetioLocatorInlineNode*
RTINetioLocatorInlineList_findEA(struct REDAInlineList* me,
				 const struct RTINetioLocator* locatorIn);

extern RTINetioDllExport struct RTINetioLocatorInlineNode *
RTINetioLocatorInlineList_addToFrontEA(
        struct REDAInlineList *me,
        struct REDAFastBufferPool *nodePoolIn,
        const struct RTINetioLocator *locatorIn);

extern RTINetioDllExport void
RTINetioLocatorInlineList_removeEA(
        struct REDAInlineList *me,
        struct REDAFastBufferPool *nodePoolIn,
        const struct RTINetioLocator *locatorIn);

extern RTINetioDllExport void
RTINetioLocatorInlineList_finalizeEA(struct REDAInlineList* me,
				     struct REDAFastBufferPool* nodePoolIn);


#define RTI_NETIO_DESTINATION_LIST_NODE_FLAGS_INITIAL_PEER 0x02
#define RTI_NETIO_DESTINATION_LIST_NODE_INVALID_IP_FLAG 0x01
#define RTI_NETIO_DESTINATION_LIST_NODE_FLAGS_DEFAULT 0x00

typedef RTI_UINT32 RTINetioDestinationListFlags;

struct RTINetioDestinationListNode {
    struct RTINetioLocatorInlineNode parent;
    /*
     * Indicates how many times a destination has been asserted to the list.
     * This increases every time we call RTINetioDestinationList_assert with
     * the same locator. This can be greater than zero if multiple remote
     * participants are using the same locator (ie if one shut down but hasn't
     * lost liveliness yet), if multiples hostnames map to the same address,
     * or if a node is an initial_peer node that has multiple WAN locators
     * that map to it. When this is 0 a node can be removed from the list.
     */
    RTI_UINT32 refcount;
    RTINetioDestinationListFlags flags;
    /*
     * Indicates how many times fully discovered remote participants are using
     * a locator in the list. This is only used for SPDP2 currently and tracks
     * when initial_peers should be disabled and re-enabled.
     *
     * When a remote participant is fully discovered we increment this count
     * and disable the destination (aka remove it from the anon writer). When
     * a remote participant stops using this locator or goes away, we decrement
     * this count. When this count goes to 0, we can re-enable the destination
     * (aka re-add it to the anon writer)
     *
     * This may be greater than 0 for a WAN initial_peer that has multiple
     * UUID+public locators associated with it, or for a locator that is
     * being used by multiple remote participants (for example if one shutdown
     * ungracefully and has not been removed but a new participant has
     * been started).
     *
     * It is also possible for the remoteParticipantRefCount to be greater than
     * 0 and for the locator to NOT be disabled. In the above shutdown scenario
     * the remoteParticipantRefCount will become 1 with SPDP2 once the first
     * participant is fully discovered. The destination will be re-enabled
     * when the second participant is discovered, and the
     * remoteParticipantRefCount will go to 2. When the second remote
     * participant is fully discovered, the destination will be disabled again
     * and the remoteParticipantRefCount will remain 2. This is so that the
     * destination is only fully re-enabled when both participants are removed
     * and the remoteParticipantRefCount goes to 0.
     */
    RTI_UINT32 remoteParticipantRefCount;
};

struct RTINetioDestinationList {
    /*
     * Inline list in which destinations are grouped based on (transport_class,
     * IP address, port). This is needed by the DNS tracker. Notice the use of
     * "grouped" instead of "ordering". The only guarantee is that all the
     * entries corresponding to the same (transport_class, IP address) will be
     * together.
     */
    struct REDAInlineList list;
    /*
     * Ordered inline list of named destinations based on (transport_class, IP
     * address, port). This is updated when the DNS
     * tracker resolves a new hostname. Each entry in this list can correspond
     * to an entry in the list (not to multiple), but multiple entries in this
     * list can correspond to the same address in the list.
     */
    struct REDAInlineList listWithNames;
    /*
     * Skip list that contains the nodes in list. This is used to make
     * finding a node in the list faster. Any node that is in list will also
     * be in searchList. When nodes are removed from list, they are also
     * removed from searchList.
     *
     * a RTINetoLocatorCompareOptions with the following settings set to TRUE:
     * compareAddress
     * comparePort
     *
     * RTINetioDestinationListNode_compare is the compare function used.
     */
    struct REDASkiplist searchList;
    struct REDASkiplistDescription searchListDescription;
    /*
     * List that contains all locators that have been changed (ie added,
     * removed, enabled, disabled) since the last time that
     * RTINetioDestinationList_resetChangeList() was called. This is used
     * to update the participant announcement channel's list of
     * destinations without copying the entire list each time.
     */
    struct REDAInlineList changeList;
    struct REDAFastBufferPool* pool;
    struct REDAExclusiveArea *ea;
    /*
     * Pointer to configurator. The configurator and the list MUST share the
     * same EA (table EA) to prevent deadlocking when the list uses the
     * configurator's EA to get transport plugin information.
     */
    struct RTINetioConfigurator *configurator;
};


#define RTINetioDestinationList_INITIALIZER \
  { \
    REDA_INLINE_LIST_EMPTY,                /* list */ \
    REDA_INLINE_LIST_EMPTY,                /* listWithNames */ \
    REDA_SKIPLIST_INITIALIZER,             /* searchList */ \
    REDA_SKIPLIST_DESCRIPTION_INITIALIZER, /* searchListDescription */ \
    REDA_INLINE_LIST_EMPTY,                /* changeList */ \
    NULL,                                  /* pool */ \
    NULL,                                  /* ea */ \
    NULL                                   /* configurator */ \
  }

extern RTINetioDllExport
void RTINetioDestinationList_finalize(
        struct RTINetioDestinationList * me);

extern RTINetioDllExport RTIBool RTINetioDestinationList_init(
        struct RTINetioDestinationList *me,
        struct RTINetioConfigurator *configurator);

extern RTINetioDllExport RTIBool RTINetioDestinationList_assert(
        struct RTINetioDestinationList *me,
        int *newHostAdded,
        int *newHostWithNameAdded,
        const struct RTINetioLocator *destinationList,
        int destinationCount,
        RTI_UINT32 fullyDiscoveredRemoteParticipantRefCount,
        RTI_UINT32 refCount,
        RTINetioDestinationListFlags flags,
        struct REDAWorker *worker);

extern RTINetioDllExport RTIBool RTINetioDestinationList_enableDiscoveryPeer(
        struct RTINetioDestinationList *me,
        RTIBool *destinationEnabled,
        const struct RTINetioLocator *destinationList,
        int destinationCount,
        struct REDAWorker *worker);

extern RTINetioDllExport RTIBool RTINetioDestinationList_removeOrDisable(
        struct RTINetioDestinationList *me,
        RTIBool *destinationWasRemoved,
        const struct RTINetioLocator *destinationList,
        int destinationCount,
        RTIBool disableDestinationOnly,
        RTIBool forceDestinationRemoval,
        struct REDAWorker *worker);


#define RTINetioDestinationList_disable( \
        me__, \
        destinationWasRemoved__, \
        destinationList__, \
        destinationCount__, \
        worker__) \
  RTINetioDestinationList_removeOrDisable( \
          (me__), \
          (destinationWasRemoved__), /* destinationWasRemoved */ \
          (destinationList__), \
          (destinationCount__), \
          RTI_TRUE,  /* disableDestinationOnly */ \
          RTI_FALSE, /* forceDestinationRemoval */ \
          (worker__))

#define RTINetioDestinationList_remove( \
        me__, \
        destinationWasRemoved__, \
        destinationList__, \
        destinationCount__, \
        forceDestinationRemoval__, \
        worker__) \
  RTINetioDestinationList_removeOrDisable( \
          (me__), \
          (destinationWasRemoved__), /* destinationWasRemoved */ \
          (destinationList__), \
          (destinationCount__), \
          RTI_FALSE, /* disableDestinationOnly */ \
          (forceDestinationRemoval__), \
          (worker__))


extern RTINetioDllExport RTIBool
RTINetioDestinationList_removeName(
    struct RTINetioDestinationList *me,
    RTIBool *destinationWasRemoved,
    RTIBool *destinationWithNameWasRemoved,
    const struct RTINetioLocator *destinationList,
    int destinationCount,
    RTIBool forceDestinationRemoval,
    struct REDAWorker *worker);

extern RTINetioDllExport RTIBool
RTINetioDestinationList_update(
    struct RTINetioDestinationList *me,
    const struct RTINetioLocator *destinationWithName,
    const NDDS_Transport_Address_t *newAddress,
    RTIBool *destinationUpdated,
    struct REDAWorker *worker);

extern RTINetioDllExport
RTIBool
RTINetioDestinationList_startIteration(
    struct RTINetioDestinationList *me,
    struct RTINetioDestinationListNode **nodeOut,
    RTIBool useNameList,
    struct REDAWorker *worker);

extern RTINetioDllExport
RTIBool
RTINetioDestinationList_endIteration(
    struct RTINetioDestinationList *me,
    struct REDAWorker *worker);

extern RTINetioDllExport void RTINetioDestinationList_resetChangeList(
        struct RTINetioDestinationList *me);

extern RTINetioDllExport struct RTINetioDestinationListNode *
RTINetioDestinationList_findEquivalentInitialPeerEA(
        struct RTINetioDestinationList *me,
        const struct RTINetioLocator *locatorIn,
        RTIBool ignoreEqualAddress,
        struct REDAWorker *worker);

struct RTINetioLocatorInfo { 
    /*e @brief A locator. */
    struct RTINetioLocator locator;

    /*e @brief Transport specific priority */
    int transport_priority;

    /*e @brief Alias list, that specifies the list of transport plugin 
      instances that should handle the locator.      
    */
    struct RTINetioAliasList aliasList;
};

extern RTINetioDllExport 
void RTINetioLocatorInfo_print(const struct RTINetioLocatorInfo *me, 
			       const char *desc,
			       int indent);

extern RTINetioDllExport
int RTINetioLocatorInfo_compare(
        const struct RTINetioLocatorInfo *l, 
        const struct RTINetioLocatorInfo *r,
        RTIBool compareAddress);

extern RTINetioDllExport
void RTINetioLocatorInfo_from(struct RTINetioLocatorInfo *me, 
                         NDDS_Transport_ClassId_t        transportIn,
                         const NDDS_Transport_Address_t *addressIn,
                         NDDS_Transport_Port_t           portIn,
                         int transport_priorityIn,
                         const char* stringAliasListIn);


#define RTI_NETIO_LOCATOR_INFO_DEFAULT {        \
    RTI_NETIO_LOCATOR_INVALID, /* Locator */    \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_DEFAULT /* aliasList */ }


/* clang-format off */
#define RTI_NETIO_LOCATOR_INFO_INTRA {         \
    { NDDS_TRANSPORT_CLASSID_INTRA,         \
      RTI_NETIO_ADDRESS_INVALID, NULL, 0, 0, {0}, 0, RTI_FALSE }, \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_INTRA, /* aliasList */ }
/* clang-format on */


/* clang-format off */
#define RTI_NETIO_LOCATOR_INFO_SHMEM {        \
    { NDDS_TRANSPORT_CLASSID_SHMEM, \
      RTI_NETIO_ADDRESS_INVALID, NULL, 0, 0, {0}, 0, RTI_FALSE }, \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_SHMEM, /* aliasList */ }
/* clang-format on */


/* clang-format off */
#define RTI_NETIO_LOCATOR_INFO_UDPv4 {        \
    { NDDS_TRANSPORT_CLASSID_UDPv4, \
      RTI_NETIO_ADDRESS_INVALID, NULL, 0, 0, {0}, 0, RTI_FALSE }, \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_UDPv4 /* aliasList */ }
/* clang-format on */


/* clang-format off */
#define RTI_NETIO_LOCATOR_INFO_UDPv4_WAN {        \
    { NDDS_TRANSPORT_CLASSID_UDPv4_WAN, \
      RTI_NETIO_ADDRESS_INVALID, NULL, 0, 0, {0}, 0, RTI_FALSE }, \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_UDPv4_WAN, /* aliasList */ }
/* clang-format on */


/* clang-format off */
#define RTI_NETIO_LOCATOR_INFO_UDPv6 {        \
    { NDDS_TRANSPORT_CLASSID_UDPv6, \
      RTI_NETIO_ADDRESS_INVALID, NULL, 0, 0, {0}, 0, RTI_FALSE }, \
    NDDS_TRANSPORT_PRIORITY_DEFAULT, /* transport priority */  \
    RTI_NETIO_ALIAS_LIST_UDPv6, /* aliasList */ }

/* clang-format on */

struct RTINetioMessage {
    NDDS_Transport_Message_t message;
    struct RTINtpTime timestamp;
    struct NDDS_Transport_Context_t transportContext;
};


#define RTI_NETIO_MESSAGE_INVALID  { \
    NDDS_TRANSPORT_MESSAGE_INVALID, \
    RTI_NTP_TIME_ZERO, \
    NDDS_TRANSPORT_CONTEXT_INVALID \
}


struct RTINetioCapArguments {
    struct RTINetioCapManager *manager;
    struct RTINetioCapTransportGatherBuffer rtpsBuffer;
    struct RTINetioCapTransportGatherBuffer rtpsDecodedBuffer;
    struct RTINetioCapTransportGatherBuffer rtpsDecodedSubmessagesBuffer;
};


#define RTINetioCapArguments_INITIALIZER { \
    NULL, \
    RTI_NETIO_CAP_TRANSPORT_GATHER_BUFFER_INITIALIZER, \
    RTI_NETIO_CAP_TRANSPORT_GATHER_BUFFER_INITIALIZER, \
    RTI_NETIO_CAP_TRANSPORT_GATHER_BUFFER_INITIALIZER }

struct RTINetioTransportInfo {
   NDDS_Transport_ClassId_t classid;
   RTI_INT32 messageSizeMax;
};

struct RTINetioConcurrency {
    struct REDAExclusiveArea *configuratorTableEa;
    struct REDAExclusiveArea *senderTableEa;
    struct REDAExclusiveArea *receiverTableEa;
};


#define RTINetioConcurrency_INITIALIZER {NULL, NULL, NULL}


#ifdef __cplusplus
    }	/* extern "C" */
#endif

  #include "netio/netio_common_impl.h"

#endif /* netio_common_h */
