/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_ntptime.h    generated by: makeheader    Sat Mar  9 09:37:09 2024
 *
 *		built from:	ntptime.ifc
 */

#ifndef osapi_ntptime_h
#define osapi_ntptime_h



  #ifndef osapi_type_h
    #include "osapi/osapi_type.h"
  #endif

  #ifndef osapi_dll_h
    #include "osapi/osapi_dll.h"
  #endif


  #if defined(RTI_VXWORKS)
    #include "osapi/osapi_vxVersions.h"
    #if VXWORKS_VERSION_6_3_OR_BETTER
      #include <sys/time.h>
    #endif
  #elif defined(RTI_WIN32)
    #include <sys/timeb.h>
  #elif defined(RTI_UNIX) || defined(RTI_LYNX) || defined(RTI_QNX) || defined(RTI_INTIME)
      #if defined(RTI_LYNX) && !defined(RTI_LYNXOS_SE) && !defined(RTI_LYNX500)
        #include <time.h>
      #else
        #include <sys/time.h>
      #endif
  #elif defined(RTI_INTY)
    #include <INTEGRITY.h>
  #endif

  #ifdef RTI_64BIT_TIME_T
    #define RTI_TIME_T time_t
  #else
    #define RTI_TIME_T RTI_INT64
  #endif

#ifdef __cplusplus
    extern "C" {
#endif

typedef struct RTINtpTime {
    /*e Seconds.**/
    RTI_INT64 sec;
    /*e fraction of a second in 1/2^32 form.*/
    RTI_UINT32 frac;
} RTINtpTime;



/*e \ingroup RTINtpTimeClass
 * \brief The size of a serialized RTINtpTime.
 */
#define RTI_OSAPI_NTPTIME_SERIALIZED_SIZE (sizeof(RTI_INT64) + sizeof(RTI_UINT32))

/*e \ingroup RTINtpTimeClass

  Represents infinite time value. Handy for specifying no timeout or
  sleeping forever.
*/
#define RTI_NTP_TIME_INFINITE ((RTINtpTime *)NULL)

/*e \ingroup RTINtpTimeClass

  @brief Zero time.

  This global variable is for convenience. This allows users to
  check and see if a RTITime variable is negative or positive by comparing
  against this.

  @see RtiTimeCompare
*/
#define RTI_NTP_TIME_ZERO {0, 0}

/*e \ingroup RTINtpTimeClass

  Represents maximum timevalue that can be represented using the
  NTP time format. For all practical purposes, can be considered
  equivalent to infinity.

*/
#define RTI_NTP_TIME_MAX {RTI_NTP_TIME_SEC_MAX, RTI_NTP_TIME_FRAC_MAX}

/*e \ingroup RTINtpTimeClass
   The maximum number of seconds, 4294967295, that can be represented using NTP
   time. This value is the maximum range of an unsigned 32-bit.
 */
#define RTI_NTP_TIME_SEC_MAX 0x00000000FFFFFFFFLL

/*e
 * \ingroup RTINtpTimeClass
 * The minimum number of seconds that can be represented using NTP time.
 * This value, 0xFFFFFFFF00000001 represent the -4294967295 in two's complement.
 */
#define RTI_NTP_TIME_SEC_MIN ((RTI_INT64) 0xFFFFFFFF00000001)

/*e \ingroup RTINtpTimeClass
   The largest possible value of the fraction field in NTP time.
 */
#define RTI_NTP_TIME_FRAC_MAX ((RTI_UINT32) 0xffffffff)

/*e \ingroup RTINtpTimeClass

  The AUTO value semantic depends on the usage.

*/
#define RTI_NTP_TIME_AUTO {RTI_NTP_TIME_SEC_AUTO, RTI_NTP_TIME_FRAC_AUTO}

/*e \ingroup RTINtpTimeClass
   The number of seconds corresponding to RTI_NTP_TIME_AUTO.
 */
#define RTI_NTP_TIME_SEC_AUTO ((RTI_INT64) 0xffffffffffffffff)

/*e \ingroup RTINtpTimeClass
   The fraction of seconds corresponding to RTI_NTP_TIME_AUTO.
 */
#define RTI_NTP_TIME_FRAC_AUTO 0U

/*e \ingroup RTINtpTimeClass
   The number of seconds corresponding to RTI_NTP_TIME_INVALID.
 */
#define RTI_NTP_TIME_SEC_INVALID ((RTI_INT64) 0xffffffffffffffff)

/*e \ingroup RTINtpTimeClass
   The fraction of seconds corresponding to RTI_NTP_TIME_INVALID.
 */
#define RTI_NTP_TIME_FRAC_INVALID RTI_NTP_TIME_FRAC_MAX

/*e
 * \ingroup RTINtpTimeClass
 *
 *  @brief Invalid time.
 *
 * Represents an invalid timevalue that can be represented using the NTP time
 * format.
 */
#define RTI_NTP_TIME_INVALID {RTI_NTP_TIME_SEC_INVALID, RTI_NTP_TIME_FRAC_INVALID}

/*
 * The following define represents the maximum size of a string containing an
 * NTP Time. So since the NTP Time structure contains an RTI_INT64, and a
 * RTI_UINT32 attribute, the value result of the following:
 * - RTI_INT64: 1 - 19 bytes
 * - RTI_UINT32: 1 - 9 bytes
 * - 4 extra characters: '{' + ',' + '}'' + '\0'
 * 19 + 9 + 4 = 32
 */
#define RTI_NTP_TIME_STRING_LEN (32)

extern RTIOsapiDllExport void RTINtpTime_setZero(struct RTINtpTime* time);

extern RTIOsapiDllExport void RTINtpTime_setMax(struct RTINtpTime* time);

extern RTIOsapiDllExport void RTINtpTime_toDouble(
        const struct RTINtpTime *t,
        RTI_DOUBLE64 *value);

extern RTIOsapiDllExport void RTINtpTime_toUInt64(
        const struct RTINtpTime* t,
        RTI_UINT64 *value);

extern RTIOsapiDllExport void RTINtpTime_fromUInt64(
        struct RTINtpTime* t,
        RTI_UINT64 value);

extern RTIOsapiDllExport void RTINtpTime_copy(struct RTINtpTime *out, 
                                              const struct RTINtpTime *in);

extern RTIOsapiDllExport void RTINtpTime_packFromMillisec(
        struct RTINtpTime time,
        long long s,
        RTI_UINT32 msec);

extern RTIOsapiDllExport void RTINtpTime_unpackToMillisec(
        long long s,
        unsigned msec,
        struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_packFromMicrosec(
        struct RTINtpTime time,
        long long s,
        RTI_UINT32 usec);

extern RTIOsapiDllExport void RTINtpTime_unpackToMicrosec(
        long long s,
        unsigned usec,
        struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_packFromNanosec(
        struct RTINtpTime time,
        long long s,
        RTI_UINT32 nsec);

extern RTIOsapiDllExport void RTINtpTime_unpackToNanosec(long long s, unsigned nsec, struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_packFromFraction(struct RTINtpTime time,
				 RTI_UINT32 numerator,
				 RTI_UINT32 denominator_per_sec);

extern RTIOsapiDllExport void RTINtpTime_unpackToFraction(unsigned numerator,
				 unsigned denominator_per_sec,
				 struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_unpackToFractionPrecise(unsigned numerator,
					unsigned denominator_per_sec,
					struct RTINtpTime time,
					unsigned precisionBits);

extern RTIOsapiDllExport
void RTINtpTime_packFromDouble(struct RTINtpTime *time, double num);

extern RTIOsapiDllExport int RTINtpTime_compare(const struct RTINtpTime time1,
		       const struct RTINtpTime time2);

extern RTIOsapiDllExport void RTINtpTime_min(
    struct RTINtpTime answer,
    const struct RTINtpTime time1,
    const struct RTINtpTime time2);

extern RTIOsapiDllExport void RTINtpTime_max(
    struct RTINtpTime answer,
    const struct RTINtpTime time1,
    const struct RTINtpTime time2);

extern RTIOsapiDllExport void RTINtpTime_add(struct RTINtpTime answer,
		    struct RTINtpTime t1, struct RTINtpTime t2);

extern RTIOsapiDllExport void RTINtpTime_subtract(struct RTINtpTime answer,
			 struct RTINtpTime t1, struct RTINtpTime t2);

extern RTIOsapiDllExport void RTINtpTime_shiftLeft(struct RTINtpTime answer,
			  struct RTINtpTime time, int shift);

extern RTIOsapiDllExport void RTINtpTime_shiftRight(struct RTINtpTime answer,
			   struct RTINtpTime time, int shift);

extern RTIOsapiDllExport void RTINtpTime_scale(struct RTINtpTime *answer,
                                               struct RTINtpTime *time, 
                                               RTI_INT32 percentScaler);

extern RTIOsapiDllExport void RTINtpTime_divide(struct RTINtpTime *answer,
                                                struct RTINtpTime *time, 
                                                RTI_INT64 divisor);

extern RTIOsapiDllExport void RTINtpTime_increment(struct RTINtpTime answer,
			  struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_decrement(struct RTINtpTime answer,
			  struct RTINtpTime time);

extern RTIOsapiDllExport void RTINtpTime_incrementInfinitesimally(struct RTINtpTime time);

extern RTIOsapiDllExport RTIBool RTINtpTime_isInfinite(const struct RTINtpTime* time);

extern RTIOsapiDllExport RTIBool RTINtpTime_isAuto(const struct RTINtpTime* time);

extern RTIOsapiDllExport RTIBool RTINtpTime_isInvalid(const struct RTINtpTime* time);

extern RTIOsapiDllExport int RTINtpTime_compareToZero(const struct RTINtpTime* time);

extern RTIOsapiDllExport
char *RTINtpTime_toString(const struct RTINtpTime* time, char* str);

extern RTIOsapiDllExport 
void RTINtpTime_toTimeval(
        const struct RTINtpTime *time, 
        struct timeval *timeval);

extern RTIOsapiDllExport 
void RTINtpTime_toCalendarTime(
        const struct RTINtpTime *time,
        RTI_TIME_T *calendarTime);

extern RTIOsapiDllExport 
void RTINtpTime_toTimespec(
        const struct RTINtpTime *time,
        struct timespec *calendarTime);

extern RTIOsapiDllExport
void RTINtpTime_toBuffer(const struct RTINtpTime* time, char* buffer);


#ifdef __cplusplus
    }   /* extern "C" */
#endif

  #include "osapi/osapi_ntptime_impl.h"


#endif /* osapi_ntptime_h */
