/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_rtpsTime.h    generated by: makeheader    Sat Mar  9 09:37:09 2024
 *
 *		built from:	rtpsTime.ifc
 */

#ifndef osapi_rtpsTime_h
#define osapi_rtpsTime_h



  #ifndef osapi_ntptime_h
    #include "osapi/osapi_ntptime.h"
  #endif

#ifdef __cplusplus
    extern "C" {
#endif

typedef struct RTIRtpsDuration {
    /*e Seconds.**/
    RTI_INT32 sec;
    /*e fraction of a second in 1/2^32 form.*/
    RTI_UINT32 frac;
} RTIRtpsDuration;



/*e
 * \ingroup RTIRtpsClass
 *
 * The infinite number of seconds that can be represented using RTPS duration.
 * This value is defined by the standard.
 */
#define RTI_RTPS_DURATION_SEC_INFINITE 0x7fffffff

/*e
 * \ingroup RTIRtpsClass
 *
 * The infinite value of the fraction field in RTPS duration. This
 * value is defined by the standard.
 */
#define RTI_RTPS_DURATION_FRAC_INFINITE ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * Represents infinite timevalue that can be represented using the
 * RTPS duration format. This value is defined by the standard.
 */
#define RTI_RTPS_DURATION_INFINITE {RTI_RTPS_DURATION_SEC_INFINITE, RTI_RTPS_DURATION_FRAC_INFINITE}

/*e
 * \ingroup RTIRtpsClass
 *
 * The maximum number of seconds that can be represented using RTPS duration.
 */
#define RTI_RTPS_DURATION_SEC_MAX ((RTI_INT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The largest possible value of the fraction field in RTPS duration.
 */
#define RTI_RTPS_DURATION_FRAC_MAX  ((RTI_UINT32) 0xfffffffe)

/*e
 * \ingroup RTIRtpsClass
 *
 * Represents maximum timevalue that can be represented using the
 * RTPS duration format.
 */
#define RTI_RTPS_DURATION_MAX {RTI_RTPS_DURATION_SEC_MAX, RTI_RTPS_DURATION_FRAC_MAX}

/*e
 * \ingroup RTIRtpsClass
 *
 * The number of seconds corresponding to RTI_RTPS_DURATION_AUTO.
 */
#define RTI_RTPS_DURATION_SEC_AUTO ((RTI_INT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The fraction of seconds corresponding to RTI_RTPS_DURATION_AUTO.
 */
#define RTI_RTPS_DURATION_FRAC_AUTO 0U

/*e
 * \ingroup RTIRtpsClass
 *
 * The AUTO value semantic depends on the usage.
 */
#define RTI_RTPS_DURATION_AUTO {RTI_RTPS_DURATION_SEC_AUTO, RTI_RTPS_DURATION_FRAC_AUTO}

/*e
 * \ingroup RTIRtpsClass
 *
 * The number of seconds corresponding to RTI_RTPS_DURATION_INVALID. This
 * value is defined by the standard.
 */
#define RTI_RTPS_DURATION_SEC_INVALID ((RTI_INT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The fraction of seconds corresponding to RTI_RTPS_DURATION_INVALID. This
 * value is defined by the standard.
 */
#define RTI_RTPS_DURATION_FRAC_INVALID  ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The INVALID value semantic depends on the usage. This value is defined by the
 * standard.
 */
#define RTI_RTPS_DURATION_INVALID {RTI_RTPS_DURATION_SEC_INVALID, RTI_RTPS_DURATION_FRAC_INVALID}


extern RTIOsapiDllExport RTIBool RTIRtpsDuration_isInfinite(
        const struct RTIRtpsDuration *duration);

extern RTIOsapiDllExport RTIBool RTIRtpsDuration_isAuto(
        const struct RTIRtpsDuration *duration);

extern RTIOsapiDllExport RTIBool RTIRtpsDuration_isInvalid(
        const struct RTIRtpsDuration* duration);

extern RTIOsapiDllExport void RTIRtpsDuration_fromNtptime(
        struct RTIRtpsDuration *rtpsDuration,
        const struct RTINtpTime *time);

extern RTIOsapiDllExport void RTIRtpsDuration_toNtptime(
        const struct RTIRtpsDuration *rtpsDuration,
        struct RTINtpTime *time);

typedef struct RTIRtpsTime {
    /*e Seconds.**/
    RTI_UINT32 sec;
    /*e fraction of a second in 1/2^32 form.*/
    RTI_UINT32 frac;
} RTIRtpsTime;


/*e
 * \ingroup RTIRtpsClass
 *
 * The infinite number of seconds that can be represented using RTPS time. This
 * value is defined by the standard.
 */
#define RTI_RTPS_TIME_SEC_INFINITE ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The infinite value of the fraction field in RTPS time. This
 * value is defined by the standard.
 */
#define RTI_RTPS_TIME_FRAC_INFINITE ((RTI_UINT32) 0xfffffffe)

/*e
 * \ingroup RTIRtpsClass
 *
 * Represents infinite timevalue that can be represented using the
 * RTPS time format. This value is defined by the standard.
 */
#define RTI_RTPS_TIME_INFINITE {RTI_RTPS_TIME_SEC_INFINITE, RTI_RTPS_TIME_FRAC_INFINITE}

/*e
 * \ingroup RTIRtpsClass
 *
 * The maximum number of seconds that can be represented using RTPS time.
 */
#define RTI_RTPS_TIME_SEC_MAX ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The largest possible valid value of the fraction field in RTPS time.
 */
#define RTI_RTPS_TIME_FRAC_MAX ((RTI_UINT32) 0xfffffffe)

/*e
 * \ingroup RTIRtpsClass
 *
 * Represents maximum timevalue that can be represented using the
 * RTPS time format.
 */
#define RTI_RTPS_TIME_MAX {RTI_RTPS_TIME_SEC_MAX, RTI_RTPS_TIME_FRAC_MAX}

/*e
 * \ingroup RTIRtpsClass
 *
 * The number of seconds corresponding to RTI_RTPS_TIME_INVALID.
 */
#define RTI_RTPS_TIME_SEC_INVALID ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The fraction of seconds corresponding to RTI_RTPS_TIME_INVALID.
 */
#define RTI_RTPS_TIME_FRAC_INVALID ((RTI_UINT32) 0xffffffff)

/*e
 * \ingroup RTIRtpsClass
 *
 * The INVALID value semantic depends on the usage. Share the same values with
 * RTI_RTPS_TIME_AUTO.
 */
#define RTI_RTPS_TIME_INVALID {RTI_RTPS_TIME_SEC_INVALID, RTI_RTPS_TIME_FRAC_INVALID}

/*e
 * \ingroup RTIRtpsClass
 *
 * The number of seconds corresponding to RTI_RTPS_TIME_AUTO.
 */
#define RTI_RTPS_TIME_SEC_AUTO RTI_RTPS_TIME_SEC_INVALID

/*e
 * \ingroup RTIRtpsClass
 *
 * The fraction of seconds corresponding to RTI_RTPS_TIME_AUTO.
 */
#define RTI_RTPS_TIME_FRAC_AUTO RTI_RTPS_TIME_FRAC_INVALID

/*e
 * \ingroup RTIRtpsClass
 *
 * The AUTO value semantic depends on the usage. Share the same values with
 * RTI_RTPS_TIME_INVALID, when transforming this into an RTINtpTime we transform
 * this into a INVALID.
 */
#define RTI_RTPS_TIME_AUTO RTI_RTPS_TIME_INVALID


extern RTIOsapiDllExport int RTIRtpsTime_compare(
        const struct RTIRtpsTime time1,
        const struct RTIRtpsTime time2);

extern RTIOsapiDllExport RTIBool RTIRtpsTime_isInfinite(
        const struct RTIRtpsTime *time);

extern RTIOsapiDllExport RTIBool RTIRtpsTime_isInvalid(
        const struct RTIRtpsTime* time);

extern RTIOsapiDllExport void RTIRtpsTime_fromNtptime(
        struct RTIRtpsTime *rtpsTime,
        const struct RTINtpTime *time);

extern RTIOsapiDllExport void RTIRtpsTime_toNtptime(
        const struct RTIRtpsTime *rtpsTime,
        struct RTINtpTime *time);


#ifdef __cplusplus
    }   /* extern "C" */
#endif

  #include "osapi/osapi_ntptime_impl.h"
  #include "osapi/osapi_rtpsTime_impl.h"


#endif /* osapi_rtpsTime_h */
