/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_sharedMemoryPthreadMutex_impl.h    generated by: makeheader    Sat Mar  9 09:37:09 2024
 *
 *		built from:	sharedMemoryPthreadMutex_impl.ifc
 */

#ifndef osapi_sharedMemoryPthreadMutex_impl_h
#define osapi_sharedMemoryPthreadMutex_impl_h


#ifdef __cplusplus
extern "C" {
#endif
#include "osapi/osapi_type.h" /* for an RTI_UINT32 member of the handle */
#include "osapi/osapi_dll.h"  /* for RTIOsapiDllExport */
#include "osapi/osapi_semaphore.h"


struct RTIOsapiSharedMemoryPthreadMutex {
    /*i
     * The pthread mutex.
     */
    pthread_mutex_t _mutex;
    /*i
     * This member is volatile because it is used to check if the pthread mutex 
     * has been initialized without being protected by a critical section.
     *
     * The operation RTIOsapiSharedMemoryPthreadMutex_attach_os checks the
     * _initialized flag until it has a valid value or a timeout expires before 
     * returning because the mutex may be in the process of being initialized 
     * by another thread.
     *
     * If we do not mark this pointer as volatile, the compiler may preload
     * the value of _initialized in to a register and the loop will incorrectly
     * timeout because it will not see the value of _initialized changed.
     */
    volatile int _initialized;
};

struct RTIOsapiSharedMemoryPthreadMutexHandle {
    /*i 
     * The shmem mutex. This is a pointer to a structure living on shared memory 
     * that contains the actual pthread mutex and a magic cookie to prevent 
     * clients from attaching to a mutex that has been allocated but 
     * not yet initialized.
     */
    struct RTIOsapiSharedMemoryPthreadMutex *_shmMutex;

    /*i
     * Unique identifier for the shared memory mutex. This is used to
     * distinguish between different mutexes in shared memory.
     */
    int _key;
};


#define RTIOsapiSharedMemoryPthreadMutexHandle_INITIALIZER { \
    NULL, /* _shmMutex */ \
    0 /* _key */ \
}

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_create(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle,
        int *status,
        int key);

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_attach(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle,
        int *status,
        int key);

extern RTIOsapiDllExport RTIBool
RTIOsapiSharedMemoryPthreadMutex_createOrAttach(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle,
        int *status,
        int key);

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_give(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle,
        int *failReason);

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_take(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle,
        int *failReasonOut);

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_detach(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle);

extern RTIOsapiDllExport RTIBool RTIOsapiSharedMemoryPthreadMutex_delete(
        struct RTIOsapiSharedMemoryPthreadMutexHandle *handle);


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* osapi_sharedMemoryPthreadMutex_impl_h */
