/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)osapi_utility_impl.h    generated by: makeheader    Sat Mar  9 09:37:10 2024
 *
 *		built from:	utility_impl.ifc
 */

#ifndef osapi_utility_impl_h
#define osapi_utility_impl_h



#ifdef RTI_THREAD_SANITIZER_IS_ENABLED
  #define RTI_REQUIRE_NON_MANDATORY_ATOMIC_OPERATIONS
#endif

#ifdef RTI_REQUIRE_NON_MANDATORY_ATOMIC_OPERATIONS
  #include <stdatomic.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

struct RTIOsapiInlineListNode {
    struct RTIOsapiInlineListNode *next;
    struct RTIOsapiInlineListNode *prev;
};

struct RTIOsapiInlineList {
    struct RTIOsapiInlineListNode *first;
    struct RTIOsapiInlineListNode *last;
};


#define RTIOsapiUtility_max(l, r) (((l) > (r)) ? (l) : (r))
#define RTIOsapiUtility_min(l, r) (((l) < (r)) ? (l) : (r))
#define RTIOsapiUtility_abs(l) (((l) < 0) ? (-(l)) : (l))
#define RTIOsapiUtility_isWithinBound(n, lower, upper) (((n) >= (lower)) \
                                                        && ((n) <= (upper)))

/*
 * @brief Get the space left between length__ and max__.
 *
 * @pre: length__ and max__ must be of the same type.
 * @param length__ \b In. The current position.
 * @param max__ \b In. The max position.
 */
#define RTIOsapiUtility_getSpaceLeft(length__, max__) \
    (((max__) > (length__)) \
            ? ((max__) - (length__)) \
            : 0)

#define RTIOsapiUtility_strlen(str__) \
    ((str__) == NULL ? 0 : strlen(str__))

#if !defined(RTI_QNX) && !defined(RTI_VXWORKS) \
        && !defined(RTI_INTY_VERSION_10)       \
        && (ENABLE_FACE_COMPLIANCE == FACE_COMPLIANCE_LEVEL_NONE)
    #define RTIOsapiUtility_strnlen(str__, maxLength__) \
        ((str__) == NULL ? 0 : strnlen((str__), (maxLength__)))
#endif

#define RTIOsapiUtility_strstr(src, pattern) \
    (src == NULL || pattern == NULL ? NULL : strstr(src, pattern))

#define RTIOsapiUtility_strncmp(str1__, str2__, size__) \
    ((RTI_INT32) strncmp( \
        (str1__) == NULL ? "" : (str1__), \
        (str2__) == NULL ? "" : (str2__), \
        (size__)))

#define RTIOsapiUtility_strcmp(str1__, str2__) \
    ((RTI_INT32) strcmp( \
        (str1__) == NULL ? "" : (str1__), \
        (str2__) == NULL ? "" : (str2__)))

#ifdef RTI_WIN32
    #define RTIOsapiUtility_strcasecmp(src__, pattern__) \
        _stricmp((src__), (pattern__))
#elif !defined(RTI_VXWORKS)
    #define RTIOsapiUtility_strcasecmp(src__, pattern__) \
        strcasecmp((src__), (pattern__))
#endif

#define RTIOsapiUtility_assertSpaceLeft(length, max, bytesToAssert) \
        (RTIOsapiUtility_getSpaceLeft(length, max) >= (bytesToAssert))

#define RTIOsapiUtility_unusedParameter(parameter) (void) parameter

#ifdef RTI_PRECONDITION_TEST
  #define RTIOsapiUtility_unusedParameterInRelease(parameter__)
#else
  #define RTIOsapiUtility_unusedParameterInRelease(parameter__) \
    RTIOsapiUtility_unusedParameter(parameter__)
#endif

/**
 * Note from CORE-10720:
 * Passing NULL as a variadic argument might result in undefined behavior
 * because the definition of NULL may vary across platforms. If it's not
 * defined as ((void *) 0) the value might be copied to the stack as 4 byte
 * int, and at a later point of the execution will be copied as an 8 byte
 * pointer resulting in corrupted memory that could lead into segmentation
 * faults. For this reason we use explicitly ((void *) 0)
 *
 * In order to prevent this issue we are forcing passing the type of the
 * argument by using this macro.
 */
#define RTIOsapiUtility_nullArg() (void *) 0

/*
 * Use this macro to remove warnings in situations where 
 * a function is assigning its return value to a variable but 
 * the variable is not being used anywhere. This will produce a
 * "set-but-unused variable warning".
 * 
 * @param function The function returning a value which is not being used
 * @param type     The type of the expected return value of function
 *
 */
#define RTIOsapiUtility_unusedReturnValue(function, type) \
{                                                         \
    type unusedReturnValue;                               \
    unusedReturnValue = function;                         \
    (void)unusedReturnValue;                              \
} (void)0

/*
 * Use these macros to remove strict-aliasing warnings. You may not dereference
 * a pointer that was casted to the wrong type, but you may assign a casted
 * pointer to a pointer of the correct type and then dereference that
 * correct-type pointer. This method should be more performant than calling
 * memcpy.
 */

#define RTIOsapiUtility_assignValueFromDereference(__dst, __srcPtr, __dstType) { \
    __dstType *__srcPtrTyped = (__dstType *) (__srcPtr); \
    __dst = *__srcPtrTyped; \
}

#define RTIOsapiUtility_assignDereferenceFromValue(__dstPtr, __src, __srcType) { \
    __srcType *__dstPtrTyped = (__srcType *) (__dstPtr); \
    *__dstPtrTyped = (__src); \
}

#define RTIOsapiUtility_assignDereferenceFromDereference(__dstPtr, __srcPtr, __dstType) { \
    __dstType *__dstPtrTyped = (__dstType *) (__dstPtr); \
    __dstType *__srcPtrTyped = (__dstType *) (__srcPtr); \
    *__dstPtrTyped = *__srcPtrTyped; \
}

/* Expands to the number of elements in an array */
#define RTIOsapiUtility_arraySize(x) (sizeof(x)/sizeof(x[0]))

/* Expands to a string representation of s */
#define RTIOsapiUtility_stringify(s) RTIOsapiUtility_s(s)
#define RTIOsapiUtility_s(s) #s

/* Following CORE-7848 */
#ifdef RTI_WIN32
  #define RTIOsapiUtility_sscanf sscanf_s
#else
  #define RTIOsapiUtility_sscanf sscanf
#endif

#define RTIOsapiUtility_pointerToUInt64(pointer__) \
    ((RTI_UINT64)(size_t)(pointer__))

#define RTIOsapiUtility_sizeToInt32(dstVal__, srcVal__) \
    (((srcVal__) > RTI_INT32_MAX) ? \
        RTI_FALSE : \
        ((dstVal__) = (RTI_INT32) (srcVal__), RTI_TRUE))

#ifdef RTI_64BIT
#define RTIOsapiUtility_sizeToUInt32(dstVal__, srcVal__) \
    (((srcVal__) > RTI_UINT32_MAX) ? \
        RTI_FALSE : \
        ((dstVal__) = (RTI_UINT32) (srcVal__), RTI_TRUE))
#else
#define RTIOsapiUtility_sizeToUInt32(dstVal__, srcVal__) \
    ((dstVal__) = (srcVal__), RTI_TRUE)
#endif

#ifdef RTI_64BIT
#define RTIOsapiUtility_sizeToInt64(dstVal__, srcVal__) \
    (((srcVal__) > RTI_INT64_MAX) ? \
        RTI_FALSE : \
        ((dstVal__) = (RTI_INT64) (srcVal__), RTI_TRUE))
#else
#define RTIOsapiUtility_sizeToInt64(dstVal__, srcVal__) \
    ((dstVal__) = (RTI_INT64) (srcVal__), RTI_TRUE)
#endif

#define RTIOsapiUtility_int32ToSize(dstVal__, srcVal__) \
    (((srcVal__) < 0) ? \
        RTI_FALSE : \
        ((dstVal__) = (size_t) (srcVal__), RTI_TRUE))

#ifdef RTI_64BIT
#define RTIOsapiUtility_int64ToSize(dstVal__, srcVal__) \
    (((srcVal__) < 0) ? \
        RTI_FALSE : \
        ((dstVal__) = (size_t) (srcVal__), RTI_TRUE))
#else
#define RTIOsapiUtility_int64ToSize(dstVal__, srcVal__) \
    (((srcVal__) < 0 || (srcVal__) > RTI_UINT32_MAX) ? \
        RTI_FALSE : \
        ((dstVal__) = (size_t) (srcVal__), RTI_TRUE))
#endif

#ifdef RTI_64BIT
#define RTIOsapiUtility_uint64ToSize(dstVal__, srcVal__) \
    ((dstVal__) = (srcVal__), RTI_TRUE)
#else
#define RTIOsapiUtility_uint64ToSize(dstVal__, srcVal__) \
    (((srcVal__) > RTI_UINT32_MAX) ? \
        RTI_FALSE : \
        ((dstVal__) = (size_t) (srcVal__), RTI_TRUE))
#endif

#define RTIOsapiUtility_int64ToDoubleMacro(dstVal__, srcVal__) \
  (((srcVal__) > RTI_DOUBLE_MANT_MAX_AS_LONG_LONG) \
           ? RTI_FALSE \
           : ((*dstVal__) = (RTI_DOUBLE64) (srcVal__), RTI_TRUE))

#ifndef RTI_PRECONDITION_TEST
  #define RTIOsapiUtility_int64ToDouble(dstVal__, srcVal__) \
    RTIOsapiUtility_int64ToDoubleMacro(dstVal__, srcVal__)
#endif

#define RTIOsapiUtility_floatingPointNearlyEqual(first__, second__, epsilon__) \
  (RTIOsapiUtility_abs(first__ - second__) <= epsilon__)

#define RTIOsapiUtility_floatNearlyEqual(first__, second__) \
  RTIOsapiUtility_floatingPointNearlyEqual(first__, second__, FLT_EPSILON)

#define RTIOsapiUtility_doubleNearlyEqual(first__, second__) \
  RTIOsapiUtility_floatingPointNearlyEqual(first__, second__, DBL_EPSILON)

#ifdef RTI_REQUIRE_NON_MANDATORY_ATOMIC_OPERATIONS
  #define RTIOsapiUtility_nonMandatoryAtomicAdd(dest__, value__) \
    atomic_fetch_add(&(dest__), value__)
  #define RTIOsapiUtility_nonMandatoryAtomicSubtract(dest__, value__) \
    atomic_fetch_sub(&(dest__), value__)
  #define RTIOsapiUtility_nonMandatoryAtomicLoad(src__) atomic_load(&(src__))
  #define RTIOsapiUtility_nonMandatoryAtomicStore(dest__, value__) \
    atomic_store(&(dest__), value__)
#else
  #define RTIOsapiUtility_nonMandatoryAtomicAdd(dest__, value__) \
    (dest__) += (value__)
  #define RTIOsapiUtility_nonMandatoryAtomicSubtract(dest__, value__) \
    (dest__) -= (value__)
  #define RTIOsapiUtility_nonMandatoryAtomicLoad(src__) (src__)
  #define RTIOsapiUtility_nonMandatoryAtomicStore(dest__, value__) \
    (dest__) = (value__)
#endif



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* osapi_utility_impl_h */
