/*
 * (c) Copyright, Real-Time Innovations, 2002-2024.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

/*
 * This file was partly generated using resource.3.0/python/rti-generate-assert-macros.py.
 * See the script for more information.
 */

#ifndef rtitest_assert_impl_h
#define rtitest_assert_impl_h

#define RTITest_assertComparisonForTypeWithAction( \
        expected__, \
        actual__, \
        type__, \
        formatSpecifier__, \
        operation__, \
        failAction__) \
  { \
    type__ expectedTmp__ = (type__) (expected__); \
    type__ actualTmp__ = (type__) (actual__); \
    RTITest_debugAssert((actualTmp__) operation__ (expectedTmp__)); \
    if (!((actualTmp__) operation__(expectedTmp__))) { \
      char buffer__[255]; \
      if (RTITestLog_snprintf( \
                  buffer__, \
                  sizeof(buffer__), \
                  ": expected " #formatSpecifier__ \
                  ", got " #formatSpecifier__, \
                  expectedTmp__, \
                  actualTmp__) \
              > 0) { \
        RTITest_assertLog( \
                &RTI_TEST_LOG_EMPTY_TEMPLATE, \
                "[%s:%d] assertion failed: %s%s", \
                __FILE__, \
                __LINE__, \
                #actual__ " " #operation__ " " #expected__, \
                buffer__); \
      } \
      failAction__; \
    } \
    }

#define RTITest_assertComparisonForType( \
        expected__, \
        actual__, \
        type__, \
        formatSpecifier__, \
        operation__) \
  { \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            type__, \
            formatSpecifier__, \
            operation__, \
            goto done); \
    }


#define RTITest_assertComparisonForFloatingPointTypeWithAction( \
        expected__, \
        actual__, \
        type__, \
        operation__, \
        epsilon__, \
        failAction__) \
  { \
    type__ expectedTmp__ = (expected__); \
    type__ actualTmp__ = (actual__); \
    type__ diff__ = expectedTmp__ - actualTmp__; \
    /* After the macro is expanded we are actually comparing a string with */ \
    /* itself, so we need to avoid the Coverity defect adding this variable */ \
    const char *operationString__ = #operation__; \
    RTIBool assertResult__; \
    if (diff__ < 0) { \
      diff__ = -diff__; \
    } \
    if (strcmp(operationString__, "==") == 0) { \
      assertResult__ = diff__ <= epsilon__; \
    } else if (strcmp(operationString__, "!=") == 0) { \
      assertResult__ = diff__ > epsilon__; \
    } else if (strcmp(operationString__, "<") == 0) { \
      assertResult__ = actualTmp__ < expectedTmp__ && diff__ > epsilon__; \
    } else if (strcmp(operationString__, "<=") == 0) { \
      assertResult__ = actualTmp__ < expectedTmp__ || diff__ <= epsilon__; \
    } else if (strcmp(operationString__, ">") == 0) { \
      assertResult__ = actualTmp__ > expectedTmp__ && diff__ > epsilon__; \
    } else if (strcmp(operationString__, ">=") == 0) { \
      assertResult__ = actualTmp__ > expectedTmp__ || diff__ <= epsilon__; \
    } else { \
      assertResult__ = RTI_FALSE; \
    } \
    RTITest_debugAssert(assertResult__); \
    if (!(assertResult__)) { \
      char buffer__[255]; \
      /* We use double regardless of the type of the floating point */ \
      /* variables since float doesn't work properly with %f specifier */ \
      /* according to Coverity */ \
      if (RTITestLog_snprintf( \
                  buffer__, \
                  sizeof(buffer__), \
                  ": expected %.5g, got %.5g", \
                  (double) expectedTmp__, \
                  (double) actualTmp__) \
          > 0) { \
        RTITest_assertLog( \
                &RTI_TEST_LOG_EMPTY_TEMPLATE, \
                "[%s:%d] assertion failed: %s%s", \
                __FILE__, \
                __LINE__, \
                #actual__ " " #operation__ " " #expected__, \
                buffer__); \
      } \
      failAction__; \
    } \
  } \
  (void) 0

#define RTITest_assertComparisonForFloatingPointType( \
        expected__, \
        actual__, \
        type__, \
        operation__, \
        epsilon__) \
  { \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            type__, \
            operation__, \
            epsilon__, \
            goto done); \
    }
/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given integers (int) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given integers (int) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given integers (int) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given integers (int) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is less than another given integer (int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is less than another given integer (int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is less than or equal to another given integer (int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is less than or equal to another given integer (int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is greater than another given integer (int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is greater than another given integer (int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is greater than or equal to another given integer (int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given integer (int) is greater than or equal to another given integer (int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            int, \
            "%d", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned integers (unsigned int) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned integers (unsigned int) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned integers (unsigned int) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned integers (unsigned int) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is less than another given unsigned integer (unsigned int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is less than another given unsigned integer (unsigned int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is less than or equal to another given unsigned integer (unsigned int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is less than or equal to another given unsigned integer (unsigned int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is greater than another given unsigned integer (unsigned int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is greater than another given unsigned integer (unsigned int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is greater than or equal to another given unsigned integer (unsigned int)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsUnsignedInt(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned integer (unsigned int) is greater than or equal to another given unsigned integer (unsigned int)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsUnsignedIntWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned int, \
            "%u", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given signed characters (signed char) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given signed characters (signed char) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given signed characters (signed char) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given signed characters (signed char) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is less than another given signed character (signed char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is less than another given signed character (signed char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is less than or equal to another given signed character (signed char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is less than or equal to another given signed character (signed char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is greater than another given signed character (signed char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is greater than another given signed character (signed char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is greater than or equal to another given signed character (signed char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsSignedChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given signed character (signed char) is greater than or equal to another given signed character (signed char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsSignedCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            signed char, \
            "%c", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given characters (unsigned char) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given characters (unsigned char) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertEqualsCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given characters (unsigned char) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given characters (unsigned char) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertNotEqualsCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is less than another given character (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is less than another given character (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertLessThanCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned int) (expected__), \
            (unsigned int) (actual__), \
            int, \
            "%c", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is less than or equal to another given character (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is less than or equal to another given character (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertLessOrEqualsCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is greater than another given character (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is greater than another given character (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertGreaterThanCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned char) (expected__), \
            (unsigned char) (actual__), \
            int, \
            "%c", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is greater than or equal to another given character (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsChar(expected__, actual__) \
    RTITest_assertComparisonForType( \
            (unsigned int) (expected__), \
            (unsigned int) (actual__), \
            int, \
            "%c", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given character (unsigned char) is greater than or equal to another given character (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 */
#define RTITest_assertGreaterOrEqualsCharWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            (unsigned int) (expected__), \
            (unsigned int) (actual__), \
            int, \
            "%c", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given short integers (char) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given short integers (char) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given short integers (char) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given short integers (char) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is less than another given short integer (char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is less than another given short integer (char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is less than or equal to another given short integer (char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is less than or equal to another given short integer (char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is greater than another given short integer (char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is greater than another given short integer (char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is greater than or equal to another given short integer (char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given short integer (char) is greater than or equal to another given short integer (char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            short, \
            "%hhd", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned short integers (unsigned char) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned short integers (unsigned char) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned short integers (unsigned char) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned short integers (unsigned char) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is less than another given unsigned short integer (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is less than another given unsigned short integer (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is less than or equal to another given unsigned short integer (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is less than or equal to another given unsigned short integer (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is greater than another given unsigned short integer (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is greater than another given unsigned short integer (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is greater than or equal to another given unsigned short integer (unsigned char)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsUnsignedShort(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned short integer (unsigned char) is greater than or equal to another given unsigned short integer (unsigned char)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsUnsignedShortWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned short, \
            "%hhu", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long integers (long) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long integers (long) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long integers (long) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long integers (long) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is less than another given long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is less than another given long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is less than or equal to another given long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is less than or equal to another given long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is greater than another given long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is greater than another given long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is greater than or equal to another given long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long integer (long) is greater than or equal to another given long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%ld", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long integers (long) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long integers (long) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long integers (long) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long integers (long) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is less than another given unsigned long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is less than another given unsigned long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is less than or equal to another given unsigned long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is less than or equal to another given unsigned long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is greater than another given unsigned long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is greater than another given unsigned long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is greater than or equal to another given unsigned long integer (long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsUnsignedLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long integer (long) is greater than or equal to another given unsigned long integer (long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsUnsignedLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long, \
            "%lu", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long long integers (long long) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long long integers (long long) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long long integers (long long) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given long long integers (long long) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is less than another given long long integer (long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is less than another given long long integer (long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is less than or equal to another given long long integer (long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is less than or equal to another given long long integer (long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is greater than another given long long integer (long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is greater than another given long long integer (long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is greater than or equal to another given long long integer (long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given long long integer (long long) is greater than or equal to another given long long integer (long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            long long, \
            "%lld", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long long integers (unsigned long long) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long long integers (unsigned long long) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long long integers (unsigned long long) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given unsigned long long integers (unsigned long long) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is less than another given unsigned long long integer (unsigned long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is less than another given unsigned long long integer (unsigned long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is less than or equal to another given unsigned long long integer (unsigned long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is less than or equal to another given unsigned long long integer (unsigned long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is greater than another given unsigned long long integer (unsigned long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is greater than another given unsigned long long integer (unsigned long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is greater than or equal to another given unsigned long long integer (unsigned long long)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsUnsignedLongLong(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given unsigned long long integer (unsigned long long) is greater than or equal to another given unsigned long long integer (unsigned long long)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsUnsignedLongLongWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            unsigned long long, \
            "%llu", \
            >=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given real numbers (float) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            ==, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given real numbers (float) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            ==, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given real numbers (float) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            !=, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given real numbers (float) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            !=, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is less than another given real number (float)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            <, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is less than another given real number (float)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            <, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is less than or equal to another given real number (float)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            <=, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is less than or equal to another given real number (float)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            <=, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is greater than another given real number (float)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            >, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is greater than another given real number (float)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            >, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is greater than or equal to another given real number (float)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsFloat(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            float, \
            >=, \
            FLT_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given real number (float) is greater than or equal to another given real number (float)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsFloatWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            float, \
            >=, \
            FLT_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given double precision real numbers (double) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            ==, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given double precision real numbers (double) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            ==, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given double precision real numbers (double) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            !=, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given double precision real numbers (double) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            !=, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is less than another given double precision real number (double)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            <, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is less than another given double precision real number (double)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            <, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is less than or equal to another given double precision real number (double)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            <=, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is less than or equal to another given double precision real number (double)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            <=, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is greater than another given double precision real number (double)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            >, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is greater than another given double precision real number (double)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            >, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is greater than or equal to another given double precision real number (double)
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsDouble(expected__, actual__) \
    RTITest_assertComparisonForFloatingPointType( \
            expected__, \
            actual__, \
            double, \
            >=, \
            DBL_EPSILON);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given double precision real number (double) is greater than or equal to another given double precision real number (double)
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsDoubleWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForFloatingPointTypeWithAction( \
            expected__, \
            actual__, \
            double, \
            >=, \
            DBL_EPSILON, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given pointers (void *) are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsPtr(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            void *, \
            "%p", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given pointers (void *) are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsPtrWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            void *, \
            "%p", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given pointers (void *) are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsPtr(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            void *, \
            "%p", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given pointers (void *) are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsPtrWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            void *, \
            "%p", \
            !=, \
            failAction__);


/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given booleans are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsBoolean RTITest_assertEqualsUnsignedShort

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given numerical chars are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Conversion to int will happen since variadic parameters will be
 * promoted to int in the call to printf as the standard states
 * (argument promotion), so Coverity thinks we are passing a char where
 * an int is expected. Conversion to unsigned char is expected by
 * STR34-C: Cast characters to unsigned char before converting to
 * larger integer sizes.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsNumericalChar(expected__, actual__) \
  RTITest_assertComparisonForType( \
          (unsigned char) (expected__), \
          (unsigned char) (actual__), \
          int, \
          "%d", \
          ==)

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given size_t are equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertEqualsSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            ==);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given size_t are equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertEqualsSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            ==, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given size_t are not equal.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertNotEqualsSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            !=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that two given size_t are not equal.
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertNotEqualsSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            !=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is less than another given size_t
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessThanSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            <);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is less than another given size_t
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessThanSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            <, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is less than or equal to another given size_t
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertLessOrEqualsSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            <=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is less than or equal to another given size_t
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertLessOrEqualsSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            <=, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is greater than another given size_t
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterThanSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            >);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is greater than another given size_t
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterThanSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            >, \
            failAction__);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is greater than or equal to another given size_t
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @details Calls `goto done` if the assertion fails. Therefore, this macro requires a `done` label to be defined in the method that this assertion is called in.
 */
#define RTITest_assertGreaterOrEqualsSize_t(expected__, actual__) \
    RTITest_assertComparisonForType( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            >=);

/**
 * @ingroup RTITestAssertion
 * @brief Assert that a given size_t is greater than or equal to another given size_t
          Otherwise, run the given action.
 * @param expected__ \b In. Expected value of the variable.
 * @param actual__ \b In. Actual value of the variable.
 * @param failAction__ \b In. Action to be run if the assertion fails.
 */
#define RTITest_assertGreaterOrEqualsSize_tWithAction(expected__, actual__, failAction__) \
    RTITest_assertComparisonForTypeWithAction( \
            expected__, \
            actual__, \
            size_t, \
            "%zu", \
            >=, \
            failAction__);

#endif