/*
 * (c) Copyright, Real-Time Innovations, 2022.
 * All rights reserved.
 *
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef rtitest_log_capture_h
#define rtitest_log_capture_h

#include <stdio.h>
#include "rtitest/test_log.h"
#include "rtitest/test_dll.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @internal
 * @brief Initializes stdout capture. Stdout will be captured in the buffer
 * passed as a parameter. This buffer will be treated as a circular buffer,
 * which means that if you write more than the size, first positions will be
 * overwritten.
 * @param size. \b In. Max size of the circular buffer.
 * @return RTI_FALSE on failure, RTI_TRUE otherwise.
 */
extern RTITestDllExport RTIBool RTITestLogCapture_start(char *buffer, size_t size);

/** @internal
 * @brief Finalizes stdout capture.
 * @return RTI_FALSE on failure, RTI_TRUE otherwise.
 */
extern RTITestDllExport RTIBool RTITestLogCapture_stop(void);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* rtitest_log_capture_h */
