/*
 * (c) Copyright, Real-Time Innovations, 2002-2022.
 * All rights reserved.
 * No duplications, whole or partial, manual or electronic, may be made
 * without express written permission.  Any such copies, or
 * revisions thereof, must display this notice unaltered.
 * This code contains trade secrets of Real-Time Innovations, Inc.
 */

#ifndef rtitest_types_h
#define rtitest_types_h

/** @internal
 * @brief Defines a function as private in the current file.
 */
#ifndef RTI_PRIVATE
  #define RTI_PRIVATE static
#endif

/** @internal
 * @brief Boolean type
 */
#ifndef RTIBool
  #define RTIBool int
#endif

#ifndef RTI_TRUE
  #define RTI_TRUE 1
#endif

#ifndef RTI_FALSE
  #define RTI_FALSE 0
#endif

#ifndef NULL
  #define NULL (0)
#endif

/**
 * @defgroup RTITestReturnCodes Return Codes
 * @brief Return codes used to determine whether a test has passed or not.
 */

/**
 * @ingroup RTITestReturnCodes
 * @brief Return code used for tests
 */
#define RTITestReturnCode int

/**
 * @ingroup RTITestReturnCodes
 * @brief Use RTI_TEST_RETCODE_FAILED when a test has failed.
 */
#define RTI_TEST_RETCODE_FAILED RTI_FALSE
/**
 * @ingroup RTITestReturnCodes
 * @brief Use RTI_TEST_RETCODE_PASSED when a test has passed.
 */
#define RTI_TEST_RETCODE_PASSED RTI_TRUE
/** @internal
 * @ingroup RTITestReturnCodes
 * @brief Use RTI_TEST_RETCODE_PASSED_ON_RETRY when a test has passed after a
 * retry.
 */
#define RTI_TEST_RETCODE_PASSED_ON_RETRY 2
/**
 * @ingroup RTITestReturnCodes
 * @brief Use RTI_TEST_RETCODE_UNSUPPORTED when a test is unsupported (e.g. a
 * file system is needed but the current platform doesn't have it).
 */
#define RTI_TEST_RETCODE_UNSUPPORTED 3
/** @internal
 * @ingroup RTITestReturnCodes
 * @brief Use RTI_TEST_RETCODE_UNIMPLEMENTED when a test has not yet been
 * implemented, and a new Connext DDS release should not be shipped without it.
 *
 * Note that every merge request should have tests that check that the behavior
 * is correct. However, in some scenarios, tests depend on changes that are not
 * yet available. You should write the test but protected with preprocessor
 * macros. Until the Connext version is updated the test would fail, so instead
 * we just return RTI_TEST_RETCODE_UNIMPLEMENTED. This works as a reminder that
 * the test has to be fully functional before shipping the next release.
 */
#define RTI_TEST_RETCODE_UNIMPLEMENTED 4

/** @internal
 * @brief Returns the function name within the current context.
 */
#if defined(RTI_WIN32) || defined(RTI_INTIME) || defined(RTI_INTY) \
        || defined(__DCC__)
  #define RTI_FUNCTION_NAME __FUNCTION__
#else
  #define RTI_FUNCTION_NAME __func__
#endif

#endif /* rtitest_types_h */
