/*

 (c) Copyright, Real-Time Innovations, 2011-2016.
 All rights reserved.
 No duplications, whole or partial, manual or electronic, may be made
 without express written permission.  Any such copies, or
 revisions thereof, must display this notice unaltered.
 This code contains trade secrets of Real-Time Innovations, Inc.

Description
-----------
Properties for the user to configure logging plugin.
=========================================================================*/
#ifndef security_logging_h
#define security_logging_h

#ifndef security_infrastructure_h
  #include "security/security_infrastructure.h"
#endif

#ifndef security_default_h
  #include "security/security_default.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif

/* General properties */
#define RTI_SECURITY_LOGGING_PROPERTY_PREFIX "logging."
#define RTI_SECURITY_VERBOSITY_PROPERTY \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "verbosity"
#define RTI_SECURITY_VERBOSITY_PROPERTY_DEFAULT "ERROR"
#define RTI_SECURITY_MODE_MASK_PROPERTY \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "mode_mask"
#define RTI_SECURITY_MODE_MASK_PROPERTY_DEFAULT "BUILTIN"

/* Security topic properties (distribution) */
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "security_topic."
#define RTI_SECURITY_SECURITY_TOPIC_PROFILE_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_PROPERTY_PREFIX \
    "profile"
#define RTI_SECURITY_SECURITY_TOPIC_AUTO_ENABLE_WRITER_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_PROPERTY_PREFIX \
    "auto_enable_writer"

/* Logging queue properties */
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_PROPERTY_PREFIX "queue."
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_SIZE_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_PROPERTY_PREFIX \
    "size"
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_MESSAGE_COUNT_MAX_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_PROPERTY_PREFIX \
    "message_count_max"
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_MESSAGE_SIZE_MAX_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_QUEUE_PROPERTY_PREFIX \
    "message_size_max"

/* Logging thread allocation thresholds */
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_PROPERTY_PREFIX "thread."
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_MESSAGE_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PROPERTY_PREFIX \
    "message_threshold"
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PLUGIN_METHOD_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PROPERTY_PREFIX \
    "plugin_method_threshold"
#define RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PLUGIN_CLASS_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_SECURITY_TOPIC_THREAD_PROPERTY_PREFIX \
    "plugin_class_threshold"

/*
 * The following properties are deprecated and will be removed in a future
 * release
 */

/* General properties */
#define RTI_SECURITY_LOG_LEVEL_PROPERTY \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "log_level"
#define RTI_SECURITY_LOG_FILE_PROPERTY \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "log_file"

/* Distribution properties */
#define RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_PROPERTY_PREFIX "distribute."
#define RTI_SECURITY_DISTRIBUTE_ENABLE_PROPERTY \
    RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX "enable"
#define RTI_SECURITY_DISTRIBUTE_PROFILE_PROPERTY \
    RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX "profile"
#define RTI_SECURITY_DISTRIBUTE_AUTO_ENABLE_WRITER_PROPERTY \
    RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX "auto_enable_writer"

/* Logging queue properties */
#define RTI_SECURITY_LOGGING_QUEUE_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX "queue."
#define RTI_SECURITY_QUEUE_SIZE_PROPERTY \
    RTI_SECURITY_LOGGING_QUEUE_PROPERTY_PREFIX "size"
#define RTI_SECURITY_MESSAGE_COUNT_MAX_PROPERTY \
    RTI_SECURITY_LOGGING_QUEUE_PROPERTY_PREFIX "message_count_max"
#define RTI_SECURITY_MESSAGE_SIZE_MAX_PROPERTY \
    RTI_SECURITY_LOGGING_QUEUE_PROPERTY_PREFIX "message_size_max"

/* Logging thread allocation thresholds */
#define RTI_SECURITY_LOGGING_THREAD_PROPERTY_PREFIX \
    RTI_SECURITY_LOGGING_DISTRIBUTE_PROPERTY_PREFIX "thread."
#define RTI_SECURITY_MESSAGE_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_THREAD_PROPERTY_PREFIX "message_threshold"
#define RTI_SECURITY_PLUGIN_METHOD_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_THREAD_PROPERTY_PREFIX "plugin_method_threshold"
#define RTI_SECURITY_PLUGIN_CLASS_THRESHOLD_PROPERTY \
    RTI_SECURITY_LOGGING_THREAD_PROPERTY_PREFIX "plugin_class_threshold"

/* Defaults */
#define RTI_SECURITY_LOGGING_LOG_LEVEL_DEFAULT DDS_LOGGING_ERROR_LEVEL
#define RTI_SECURITY_LOGGING_MESSAGE_THRESHOLD_DEFAULT       256
#define RTI_SECURITY_LOGGING_PLUGIN_METHOD_THRESHOLD_DEFAULT 80
#define RTI_SECURITY_LOGGING_PLUGIN_CLASS_THRESHOLD_DEFAULT  32
#define RTI_SECURITY_LOGGING_MESSAGE_COUNT_MAX_DEFAULT       64
#define RTI_SECURITY_LOGGING_STRING_BUFFER_SIZE_MAX          256
#define RTI_SECURITY_LOGGING_MESSAGE_SIZE_MAX_DEFAULT \
    RTI_SECURITY_LOGGING_QUEUE_MESSAGE_MAX_SIZE_DEFAULT
#define RTI_SECURITY_LOGGING_QUEUE_SIZE_DEFAULT \
    (RTI_SECURITY_LOGGING_MESSAGE_COUNT_MAX_DEFAULT \
    * RTI_SECURITY_LOGGING_MESSAGE_SIZE_MAX_DEFAULT)
#define RTI_SECURITY_LOGGING_MODE_MASK_DEFAULT RTI_SECURITY_LOG_MODE_MASK_BUILTIN


#define RTI_SECURITY_LOGGING_LOG_OPTIONS_DEFAULT { \
        RTI_SECURITY_LOGGING_LOG_LEVEL_DEFAULT, /* log_level */ \
        RTI_SECURITY_LOGGING_MODE_MASK_DEFAULT, /* mode_mask */ \
        RTI_SECURITY_LOGGING_QUEUE_SIZE_DEFAULT, /* queue_size */ \
        RTI_SECURITY_LOGGING_MESSAGE_COUNT_MAX_DEFAULT, /* queue_message_count_max */ \
        RTI_SECURITY_LOGGING_MESSAGE_SIZE_MAX_DEFAULT, /* queue_message_size_max */ \
        RTI_SECURITY_LOGGING_MESSAGE_THRESHOLD_DEFAULT, /* thread_message_threshold */ \
        RTI_SECURITY_LOGGING_PLUGIN_METHOD_THRESHOLD_DEFAULT, /* thread_plugin_method_threshold */ \
        RTI_SECURITY_LOGGING_PLUGIN_CLASS_THRESHOLD_DEFAULT, /* thread_plugin_class_threshold */ \
        NULL, /* profile */ \
        DDS_BOOLEAN_TRUE /* auto_enable_writer */ \
}

/* Keep these logging messages in alphabetical order. */
#define RTI_SECURITY_LOGGING_MESSAGE_CHALLENGE_SIGNATURE_VERIFICATION_FAILURE \
    "failed to verify challenge signature"

#define RTI_SECURITY_LOGGING_MESSAGE_DECRYPT_FINAL_FAILURE \
    "DecryptFinal failed. Possible GCM authentication failure."

#define RTI_SECURITY_LOGGING_MESSAGE_DECRYPT_FINAL_PSK_FAILURE \
    "DecryptFinal failed. Possible GCM authentication failure. " \
    "Please check that the DomainParticipants in your system are " \
    "all using the same value for the " \
    DDS_SECURITY_CRYPTOGRAPHY_RTPS_PSK_SECRET_PASSPHRASE_PROPERTY " property."

#define RTI_SECURITY_LOGGING_MESSAGE_GET_ENDPOINT_SEC_ATTRIBUTES \
    "error getting the local endpoint security attributes"

#define RTI_SECURITY_LOGGING_MESSAGE_HANDSHAKE_REPLY_SUCCESS \
    "successfully began handshake reply"

#define RTI_SECURITY_LOGGING_MESSAGE_ID_CERT_CHANGE_SUCCESS \
    "successfully changed identity certificate"

#define RTI_SECURITY_LOGGING_MESSAGE_ID_PRESHARED_SEED_CHANGE_SUCCESS \
    "successfully updated the pre-shared key seed"

#define RTI_SECURITY_LOGGING_MESSAGE_INCONSISTENT_PROTECTION_KIND_AND_MAX_RECEIVER_SPECIFIC_MACS \
    "inconsistent configuration: protection_kind has WITH_ORIGIN_AUTHENTICATION, but " \
    "cryptography.max_receiver_specific_macs < 2"

#define RTI_SECURITY_LOGGING_MESSAGE_INCORRECT_CRC \
    "Incorrect CRC detected."

#define RTI_SECURITY_LOGGING_MESSAGE_LOOKUP_KEY_FAILURE \
    "failed to look up key in db"

#define RTI_SECURITY_LOGGING_MESSAGE_MEMORY_OPTIMIZATIONS_DISABLED \
    "crypto library allocators: encoding/decoding memory optimizations are " \
    "disabled"

#define RTI_SECURITY_LOGGING_MESSAGE_MISMATCHED_ALGORITHMS_PSK_FAILURE \
    "Mismatched algorithms. Please check that the DomainParticipants in " \
    "your system are all using the same value for the " \
    DDS_SECURITY_CRYPTOGRAPHY_RTPS_PSK_SYMMETRIC_CIPHER_ALGORITHM_PROPERTY " property."

#define RTI_SECURITY_LOGGING_MESSAGE_NEW_ID_CERT_INCOMPATIBLE \
    "new certificate is not compatible with the old one"

#define RTI_SECURITY_LOGGING_MESSAGE_NONE_PSK_PROTECTION_KIND \
    "The value of the " DDS_SECURITY_ACCESS_CONTROL_RTPS_PSK_PROTECTION_KIND_PROPERTY \
    "property is \"NONE\" but the Lightweight Security Plugins are enabled. " \
    "RTPS messages will not be protected. Consider setting the protection " \
    "kind to \"ENCRYPT\" or \"SIGN\"."

#define RTI_SECURITY_LOGGING_MESSAGE_NON_NULL_TERMINATED_CERT \
    "identity certificate binary property contains a malformed " \
    "certificate. In particular, the certificate is not properly " \
    "null terminated. This is likely caused by a non-compliant " \
    "DDS Security implementation."

#define RTI_SECURITY_LOGGING_MESSAGE_OUT_OF_MEMORY "insufficient memory"

#define RTI_SECURITY_LOGGING_MESSAGE_PRESHARED_SECRET_VALIDATION_FAILURE \
            "Validation of the secret in the " \
            DDS_SECURITY_CRYPTOGRAPHY_RTPS_PSK_SECRET_PASSPHRASE_PROPERTY \
            " property failed."

#define RTI_SECURITY_LOGGING_MESSAGE_RECEIVER_SPECIFIC_MAC_NOT_FOUND \
    "received receiver_specific_macs does not contain expected " \
    "receiver_specific_key_id. This may happen if the participant has " \
    "received a directed message that was intended for another participant "\
    "on the same machine."

#define RTI_SECURITY_LOGGING_MESSAGE_WRONG_INFO_SRC \
    "received unacceptable RTPS message (wrong INFO_SRC)"

#define RTI_SECURITY_LOGGING_MESSAGE_WRONG_PRESHARED_KEY_FORMAT \
    DDS_SECURITY_CRYPTOGRAPHY_RTPS_PSK_SECRET_PASSPHRASE_PROPERTY \
    " must follow either one of two formats: (1) \"data:,<ID>:<SEED>\", " \
    "where <ID> is a unique number identifying the <SEED> (between 0 and " \
    "254) or (2) \"file:<PATH>\", where <PATH> is a path to a file " \
    "containing a string in format \"<ID>:<SEED>\"."

#define RTI_SECURITY_LOGGING_MESSAGE_WRONG_PRESHARED_ID_RANGE \
    "The seed id must be a value in the [0, 254] range"

#ifdef __cplusplus
    }   /* extern "C" */
#endif

#endif /* security_logging_h */
