/* (c) Copyright 2003-2021, Real-Time Innovations, Inc. All rights reserved. */
/*
 * @(#)transport_common.h    generated by: makeheader    Sat Mar  9 09:38:04 2024
 *
 *		built from:	common.ifc
 */

#ifndef transport_common_h
#define transport_common_h


#ifndef osapi_type_h
#include "osapi/osapi_type.h"
#endif

#ifndef osapi_rtpsGuid_h
#include "osapi/osapi_rtpsGuid.h"
#endif

#ifndef reda_buffer_h
#include "reda/reda_buffer.h"
#endif

#ifndef transport_dll_h
#include "transport/transport_dll.h"
#endif

/* Include the APIs useful for the transport plugin user */
#ifndef transport_common_user_h
  #include "transport/transport_common_user.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif



/* Data submessage ID */
#define NDDS_TRANSPORT_UDP_DATA_SUBMESSAGE_ID (0x15)

/* OID for SPDP participant announcement writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SPDP_PARTICIPANT_WRITER (0x000100C2)

/* OID for SPDP2 bootstrap participant announcement writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SPDP2_PARTICIPANT_BOOTSTRAP_WRITER \
  (0x00010082)

/* OID for SPDP2 config participant writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SPDP2_PARTICIPANT_CONFIG_WRITER \
  (0x00010182)

#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SPDP2_PARTICIPANT_SECURE_CONFIG_WRITER \
 (0xFF010182)

/* OID for publications writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_PUB_WRITER (0x000003C2)

/* OID for subscriptions writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SUB_WRITER (0x000004C2)

/* OID for publications reader */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_PUB_READER (0x000003C7)

/* OID for subscriptions reader */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SUB_READER (0x000004C7)

/* OID for liveliness writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_P2P_PARTICIPANT_LIVELINESS_WRITER \
  (0x000200C2)

/* OID for secure liveliness writer */
#define NDDS_TRANSPORT_RTPS_OBJECT_ID_SECURE_P2P_PARTICIPANT_LIVELINESS_WRITER \
  (0xFF0200C2)

/* Auto liveliness kind */
#define NDDS_TRANSPORT_INTER_PARTICIPANT_MESSAGE_KIND_AUTOMATIC_LIVELINESS_UPDATE \
  (0x00000001)

/* Manual livelienss kind */
#define NDDS_TRANSPORT_INTER_PARTICIPANT_MESSAGE_KIND_MANUAL_LIVELINESS_UPDATE \
  (0x00000002)

typedef RTI_UINT32 NDDS_Transport_Port_t;


#define NDDS_TRANSPORT_PORT_INVALID ((NDDS_Transport_Port_t) 0) 


struct NDDS_Transport_SharedPort_t {
    NDDS_Transport_Port_t port;
    int ref_count;
};


#define NDDS_TRANSPORT_UUID_SIZE 12


#define NDDS_TRANSPORT_LENGTH_UNLIMITED -1


#define NDDS_TRANSPORT_INTERFACE_RANK_UNKNOWN 0


struct NDDS_Transport_UUID {
    unsigned char value[NDDS_TRANSPORT_UUID_SIZE];
};


#define NDDS_TRANSPORT_UUID_UNKNOWN {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}

 
typedef enum {
  /*e
    \brief The transport interface is OFF.
   */
  NDDS_TRANSPORT_INTERFACE_OFF  = 0,
  /*e 
    \brief The transport interface is ON.
   */
  NDDS_TRANSPORT_INTERFACE_ON   = 1
} NDDS_Transport_Interface_Status_t;

 
typedef struct {
     /*e
       \brief The transport classid of the interface.
      */
     NDDS_Transport_ClassId_t transport_classid;
     
     /*e 
       \brief An unicast address that uniquely identifies this 
       interface in the network specified by the transport class.
      */    
     NDDS_Transport_Address_t address; 

    /*e
      \brief The state of the interface
     */
    NDDS_Transport_Interface_Status_t status;

    /*e
     * \brief Rank of the interface. Used when allow_interfaces_list Qos is set.
     * A rank value will be assing to each of the interfaces that match the
     * allow_interfaces_list filter allowing an endpoint to prioritace some
     * interfaces upon others.
     */
    RTI_UINT16 rank;
     
} NDDS_Transport_Interface_t;


#define NDDS_TRANSPORT_INTERFACE_INITIALIZER \
{ \
    NDDS_TRANSPORT_CLASSID_INVALID, \
    NDDS_TRANSPORT_ADDRESS_INVALID_INITIALIZER, \
    NDDS_TRANSPORT_INTERFACE_OFF, \
    NDDS_TRANSPORT_INTERFACE_RANK_UNKNOWN \
}

extern NDDS_Transport_DllExport
void NDDS_Transport_Interface_print(
    const NDDS_Transport_Interface_t *interface_in, 
    const char                       *desc_in, 
    RTI_INT32                         indent_in);

extern NDDS_Transport_DllExport
void NDDS_Transport_Interface_copy(
        NDDS_Transport_Interface_t *dst_out,
        const NDDS_Transport_Interface_t *src_in);


#define NDDS_TRANSPORT_CONTEXT_INVALID \
{ \
    NDDS_TRANSPORT_ADDRESS_INVALID_INITIALIZER, \
    NDDS_TRANSPORT_ADDRESS_INVALID_INITIALIZER, \
    NDDS_TRANSPORT_PORT_INVALID,    \
    NDDS_TRANSPORT_PORT_INVALID,    \
    NDDS_TRANSPORT_PORT_INVALID,    \
    NDDS_TRANSPORT_CLASSID_INVALID  \
}


struct NDDS_Transport_Context_t {
    /*i
     * @brief Source address of the message.
     */
    NDDS_Transport_Address_t sourceAddress;
    /*i
     * @brief Destination address of the message.
     */
    NDDS_Transport_Address_t destinationAddress;
    /*i
     * @brief source port of the message.
     */
    NDDS_Transport_Port_t    sourcePort;
    /*i
     * @brief Destination port of the message.
     * In the case of TCP, if participant is acting as server,
     * this is the server bind port.
     */
    NDDS_Transport_Port_t    destinationPort;
    /*i
     * @brief RTPS port when TCP and participant acting as server.
     * Otherwise, destinationRTPSPort is equal to 0 and we are assuming
     * destinationRTPSPort==destinationPort.
     */
    NDDS_Transport_Port_t    destinationRTPSPort;
    /*i
     * @brief Transport class Id. This information is filled by netio.
     */
    NDDS_Transport_ClassId_t classId;
};


#define NDDS_TRANSPORT_PRIORITY_DEFAULT (0)

typedef struct REDABuffer NDDS_Transport_Buffer_t;


#define NDDS_Transport_Buffer_t_compare(transportBuffer1, transportBuffer2) \
      (((transportBuffer1)->length < (transportBuffer2)->length) ? -1 : \
       (((transportBuffer1)->length > (transportBuffer2)->length) ?  1 : \
        (((transportBuffer1)->pointer < (transportBuffer2)->pointer) ? -1 : \
         (((transportBuffer1)->pointer > (transportBuffer2)->pointer) ? 1 : 0 ))))


#ifdef __cplusplus
    }	/* extern "C" */
#endif

  #include "transport/transport_common_impl.h"

#endif /* transport_common_h */
