# (c) 2018 Copyright, Real-Time Innovations, Inc.  All rights reserved.
# No duplications, whole or partial, manual or electronic, may be made
# without express written permission.  Any such copies, or revisions thereof,
# must display this notice unaltered.
# This code contains trade secrets of Real-Time Innovations, Inc.

#[[.rst:
.. _connextdds_log:

ConnextDds
----------

connextdds_log_error
^^^^^^^^^^^^^^^^^^^^

Prints an error message and stops the CMake execution::

    connextdds_log_error(
        MESSAGE <message>
        TYPE <type>
    )

The printed message follows this format::
    [rti::error::<type>]

Arguments:

``MESSAGE`` (mandatory).
    Message to show.

``TYPE`` (mandatory).
    Error type to log.


connextdds_unsupported_link_mode
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

Prints an error message for a unsupported linking mode::

    connextdds_unsupported_link_mode()


connextdds_log_unsupported_licensed_mode
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

Prints an error message for unsupported licensed build mode::

    connextdds_log_unsupported_licensed_mode()


connextdds_unsupported_generator
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

Prints an error message for unsupported CMake generators::

    connextdds_unsupported_generator()

#]]

include(CMakeParseArguments)
include(ConnextDdsArgumentChecks)
include(ConnextDdsLogMessage)

# File used to exchange detailed information with the automation
set(
    CONNEXTDDS_BUILD_CONFIGURATION_ERROR_FILE
    "${CMAKE_BINARY_DIR}/UNSUPPORTED_CONFIGURATION"
)


function(connextdds_log_error)
    set(single_value_args MESSAGE TYPE)
    cmake_parse_arguments(
        _LOG
        "" "${single_value_args}" ""
        ${ARGN}
    )
    connextdds_log_message(
        FATAL_ERROR "[rti::error::${_LOG_TYPE}] ${_LOG_MESSAGE}")

endfunction()

function(connextdds_unsupported_link_mode)
    if(${BUILD_SHARED_LIBS})
        set(link_mode "dynamic")
    else()
        set(link_mode "static")
    endif()


    set(message "The architecture does not support ${link_mode} builds")
    file(WRITE "${CONNEXTDDS_BUILD_CONFIGURATION_ERROR_FILE}" ${message})
    connextdds_log_error(
            MESSAGE "${message}"
            TYPE "unsupported-link-mode")
endfunction()

function(connextdds_unsupported_licensed_mode)
    set(message "The architecture does not support licensed builds")


    file(WRITE "${CONNEXTDDS_BUILD_CONFIGURATION_ERROR_FILE}" ${message})
    connextdds_log_error(
        MESSAGE "${message}"
        TYPE "unsupported-licensed-mode")
endfunction()

function(connextdds_unsupported_generator)
    set(message
        "The architecture does not support the generator "
        "${CMAKE_GENERATOR}")


    file(WRITE "${CONNEXTDDS_BUILD_CONFIGURATION_ERROR_FILE}" ${message})
    connextdds_log_error(
        MESSAGE "${message}"
        TYPE "generator")
endfunction()
